Package: dtlg
Title: A Performance-Focused Package for Clinical Trial Tables
Version: 0.1.0
Authors@R: c(
    person(
      given = "Max", family = "Ebenezer-Brown",
      email = "max.ebenezer-brown@acuityanalytics.com",
      role = "aut"
    ),
    person(
      given = "Max", family = "Norman",
      role = "aut"
    ),
    person(
      given = "Xinye", family = "Li",
      email = "xinye.li@acuityanalytics.com",
      role = "aut"
    ),
    person(
      given = "Anja", family = "Peebles-Brown",
      email = "anna.peebles-brown@acuityanalytics.com",
      role = "aut"
    ),
    person(
      given = "Ashley", family = "Baldry",
      email = "ashley.baldry@acuityanalytics.com",
      role = "aut"
    ),    
    person(
      given = "Ramiro", family = "Magno",
      email = "ramiro.morgado@acuityanalytics.com",
      role = c("aut", "cre")
    )
    )
Description: Create high-performance clinical reporting tables (TLGs) from
    ADaM-like inputs. The package provides a consistent, programmatic API
    to generate common tables such as demographics, adverse event incidence,
    and laboratory summaries, using 'data.table' for fast aggregation over
    large populations. Functions support flexible target-variable selection,
    stratification by treatment, and customizable summary statistics, and
    return tidy, machine-readable results ready to render with downstream
    table/formatting packages in analysis pipelines.
License: MIT + file LICENSE
Imports: data.table (>= 1.18.0), vctrs
Suggests: bench, dplyr, kableExtra, rmarkdown, random.cdisc.data,
        rtables, tern, testthat (>= 3.0.0), tidyr, withr
Encoding: UTF-8
RoxygenNote: 7.3.3
Depends: R (>= 4.1.0)
Config/testthat/edition: 3
Config/Needs/website: rmarkdown, ascentsoftware/ascentdown
LazyData: true
URL: https://AscentSoftware.github.io/dtlg/
BugReports: https://github.com/AscentSoftware/dtlg/issues
NeedsCompilation: no
Packaged: 2026-04-28 11:28:54 UTC; rmagno
Author: Max Ebenezer-Brown [aut],
  Max Norman [aut],
  Xinye Li [aut],
  Anja Peebles-Brown [aut],
  Ashley Baldry [aut],
  Ramiro Magno [aut, cre]
Maintainer: Ramiro Magno <ramiro.morgado@acuityanalytics.com>
Repository: CRAN
Date/Publication: 2026-04-29 09:10:07 UTC
Built: R 4.5.2; ; 2026-05-01 17:32:18 UTC; unix

Package: dtlg
Title: A Performance-Focused Package for Clinical Trial Tables
Version: 0.1.0
Authors@R: c(
    person(
      given = "Max", family = "Ebenezer-Brown",
      email = "max.ebenezer-brown@acuityanalytics.com",
      role = "aut"
    ),
    person(
      given = "Max", family = "Norman",
      role = "aut"
    ),
    person(
      given = "Xinye", family = "Li",
      email = "xinye.li@acuityanalytics.com",
      role = "aut"
    ),
    person(
      given = "Anja", family = "Peebles-Brown",
      email = "anna.peebles-brown@acuityanalytics.com",
      role = "aut"
    ),
    person(
      given = "Ashley", family = "Baldry",
      email = "ashley.baldry@acuityanalytics.com",
      role = "aut"
    ),    
    person(
      given = "Ramiro", family = "Magno",
      email = "ramiro.morgado@acuityanalytics.com",
      role = c("aut", "cre")
    )
    )
Description: Create high-performance clinical reporting tables (TLGs) from
    ADaM-like inputs. The package provides a consistent, programmatic API
    to generate common tables such as demographics, adverse event incidence,
    and laboratory summaries, using 'data.table' for fast aggregation over
    large populations. Functions support flexible target-variable selection,
    stratification by treatment, and customizable summary statistics, and
    return tidy, machine-readable results ready to render with downstream
    table/formatting packages in analysis pipelines.
License: MIT + file LICENSE
Imports: data.table (>= 1.18.0), vctrs
Suggests: bench, dplyr, kableExtra, rmarkdown, random.cdisc.data,
        rtables, tern, testthat (>= 3.0.0), tidyr, withr
Encoding: UTF-8
RoxygenNote: 7.3.3
Depends: R (>= 4.1.0)
Config/testthat/edition: 3
Config/Needs/website: rmarkdown, ascentsoftware/ascentdown
LazyData: true
URL: https://AscentSoftware.github.io/dtlg/
BugReports: https://github.com/AscentSoftware/dtlg/issues
NeedsCompilation: no
Packaged: 2026-04-28 11:28:54 UTC; rmagno
Author: Max Ebenezer-Brown [aut],
  Max Norman [aut],
  Xinye Li [aut],
  Anja Peebles-Brown [aut],
  Ashley Baldry [aut],
  Ramiro Magno [aut, cre]
Maintainer: Ramiro Magno <ramiro.morgado@acuityanalytics.com>
Repository: CRAN
Date/Publication: 2026-04-29 09:10:07 UTC
Built: R 4.6.0; ; 2026-04-29 11:07:22 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(calc_stats,character)
S3method(calc_stats,default)
S3method(calc_stats,factor)
S3method(calc_stats,logical)
S3method(calc_stats,numeric)
export(AET01_table)
export(AET02_table)
export(as_dtlg_table)
export(calc_counts)
export(calc_desc)
export(calc_stats)
export(cross_tab_to_obsv_tab)
export(dt_copy_semantics)
export(event_count)
export(event_count_by)
export(format_n_pct)
export(indent)
export(label)
export(maybe_copy_dt)
export(merge_table_lists)
export(multi_event_true)
export(nbsp)
export(print_dtlg)
export(round_pct)
export(round_sum)
export(set_dt_copy_semantics)
export(summary_table)
export(summary_table_by)
export(summary_table_by_targets)
export(tern_AET01_table)
export(tern_AET02_table)
export(tern_summary_table)
export(total_events)
export(with_label)
importFrom(data.table,":=")
importFrom(data.table,.N)
importFrom(data.table,.SD)

# dtlg

<!-- badges: start -->

[![CRAN
version](https://www.r-pkg.org/badges/version/dtlg)](https://cran.r-project.org/package=dtlg)
[![R-CMD-check](https://github.com/AscentSoftware/dtlg/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/AscentSoftware/dtlg/actions/workflows/R-CMD-check.yaml)
[![License:
MIT](https://img.shields.io/badge/license-MIT-blue.svg)](https://cran.r-project.org/web/licenses/MIT)
[![CRAN downloads last
month](http://cranlogs.r-pkg.org/badges/last-month/dtlg)](https://cran.r-project.org/package=dtlg)
<!-- badges: end -->

Create high-performance clinical reporting tables (TLGs) from ADaM-like
inputs. The package provides a consistent, programmatic API to generate
common tables such as demographics, adverse event incidence, and
laboratory summaries, using ‘data.table’ for fast aggregation over large
populations. Functions support flexible target-variable selection,
stratification by treatment, and customizable summary statistics, and
return tidy, machine-readable results ready to render with downstream
table/formatting packages in analysis pipelines.

## Installation

``` r
install.packages("dtlg")
```

## Example

Creating an AET01 table:

``` r
library(dtlg)

head(adsl[, c("USUBJID", "ARM")])
#>                 USUBJID            ARM
#> 1  AB12345-CHN-3-id-128      A: Drug X
#> 2 AB12345-CHN-15-id-262 C: Combination
#> 3  AB12345-RUS-3-id-378 C: Combination
#> 4 AB12345-CHN-11-id-220     B: Placebo
#> 5  AB12345-CHN-7-id-267     B: Placebo
#> 6 AB12345-CHN-15-id-201 C: Combination

head(aesi[, c("USUBJID", "ARM", "FATAL")])
#>                 USUBJID            ARM  FATAL
#>                  <char>         <fctr> <lgcl>
#> 1: AB12345-BRA-1-id-134      A: Drug X  FALSE
#> 2: AB12345-BRA-1-id-134      A: Drug X  FALSE
#> 3: AB12345-BRA-1-id-134      A: Drug X  FALSE
#> 4: AB12345-BRA-1-id-134      A: Drug X   TRUE
#> 5: AB12345-BRA-1-id-141 C: Combination  FALSE
#> 6: AB12345-BRA-1-id-141 C: Combination  FALSE

AET01 <-
  AET01_table(
    adsl = adsl,
    adae = aesi,
    patient_var = "USUBJID",
    treat_var = "ARM",
    aesi_vars = c("FATAL", "SER", "SERWD", "SERDSM", "RELSER",
                  "WD", "DSM", "REL", "RELWD", "RELDSM", "SEV")
  )

AET01[, stats := strtrim(stats, 35)][]
#>                                   stats   A: Drug X B: Placebo C: Combination
#>                                  <char>      <char>     <char>         <char>
#>  1: Total number of patients with at le 100 (74.6%) 98 (73.1%)    103 (78.0%)
#>  2:                 Total number of AEs         502        480            604
#>  3:              Total number of deaths  25 (18.7%) 23 (17.2%)     22 (16.7%)
#>  4: Total number of patients withdrawn     3 (2.2%)   6 (4.5%)       5 (3.8%)
#>  5: Total number of patients with at le                                      
#>  6:               AE with fatal outcome    5 (3.7%)   5 (3.7%)       6 (4.5%)
#>  7:                          Serious AE  85 (63.4%) 80 (59.7%)     87 (65.9%)
#>  8:   Serious AE leading to withdrawal     6 (4.5%)  12 (9.0%)       9 (6.8%)
#>  9:   Serious AE leading to dose modifi  36 (26.9%) 40 (29.9%)     47 (35.6%)
#> 10:                  Related Serious AE  64 (47.8%) 52 (38.8%)     64 (48.5%)
#> 11:   AE leading to withdrawal from tre  20 (14.9%) 24 (17.9%)     26 (19.7%)
#> 12:   AE leading to dose modification/i  63 (47.0%) 70 (52.2%)     77 (58.3%)
#> 13:                          Related AE  86 (64.2%) 85 (63.4%)     92 (69.7%)
#> 14:   Related AE leading to withdrawal    10 (7.5%)   9 (6.7%)      12 (9.1%)
#> 15:   Related AE leading to dose modifi  44 (32.8%) 44 (32.8%)     51 (38.6%)
#> 16:   Severe AE (at greatest intensity)  77 (57.5%) 70 (52.2%)     79 (59.8%)
```

#' Generate Non-Breaking Spaces for HTML Output
#'
#' [nbsp()] generates a string of HTML non-breaking spaces (`&nbsp;`).
#'
#' @param n Number of non-breaking spaces. Defaults to `1`.
#'
#' @returns A character string containing `n` HTML non-breaking spaces
#'   (`&nbsp;`).
#'
#' @examples
#' # One non-breaking space
#' nbsp()
#' nbsp(1)
#'
#' # Several non-breaking spaces
#' nbsp(n = 2)
#' nbsp(n = 5)
#'
#' # When `n` is zero, an empty string is returned.
#' nbsp(n = 0)
#'
#' @keywords internal
#' @export
#'
nbsp <- function(n = 1L) {

  stopifnot(
    is.numeric(n),
    n >= 0L
  )

  strrep("&nbsp;", times = n)
}

#' Add indentation to strings
#'
#' [indent()] prefixes a string with a sequence of HTML non-breaking spaces,
#' to effectively work as indentation.
#'
#' @param x A character vector of strings to be _indented_.
#'
#' @param n Number of non-breaking spaces to use as indentation. If `n = 0` then
#' no indentation is performed.
#'
#' @param indentation As an alternative to the number of spaces `n` you may
#' pass the actual sequence of HTML non-breaking spaces as a string to this
#' parameter, or any other string for that matter. Defaults to an indentation
#' of four spaces (`n = 4`).
#'
#' @returns A character vector of the same length as `x`.
#'
#' @examples
#' # Default is to indent by four non-breaking spaces.
#' indent("Mean")
#'
#' # Choose a different indentation level.
#' indent("Mean", n = 2L)
#'
#' # `indent()` is vectorised over `x`
#' indent(c("Mean", "Median", "Max, Min", "Missing"))
#'
#' @keywords internal
#' @export
#'
indent <- function(x, n = 4L, indentation = nbsp(n = n)) {

  stopifnot(
    is.character(x),
    n >= 0L
  )

  paste0(indentation, x)
}

#' Create a clinical reporting table with tern/rtables
#'
#' [tern_summary_table()] is a convenience wrapper around `{rtables}` and
#' `{tern}` commands to generate a clinical reporting summary statistics tables
#' whilst using a similar interface as [summary_table()]. This can be helpful
#' for side by side comparisons of the two functions.
#'
#' @inherit summary_table
#'
#' @seealso [summary_table()]
#'
#' @examples
#' dmg_vars <- c("AGE", "RACE", "ETHNIC")
#' dmg_var_lbls <- c("Age (yr)", "Race", "Ethnicity")
#'
#' # Demographics table (DMT01)
#' tern_summary_table(
#'   adsl,
#'   target = dmg_vars,
#'   treat = 'ARM',
#'   target_name = dmg_var_lbls
#' )
#'
#' # Demographics table (DMT01) with continuous variable (e.g., BMRKR1)
#' tern_summary_table(
#'   adsl,
#'   target = c(dmg_vars, "BMRKR1"),
#'   treat = 'ARM',
#'   target_name = c(dmg_var_lbls, "Biomarker 1")
#' )
#'
#' @export
tern_summary_table <- function(dt, target, treat, target_name = target) {

  assert_pkg_is_installed("rtables")
  assert_pkg_is_installed("tern")

  dt <- tern::df_explicit_na(dt, na_level = "Missing")

  rtables::basic_table(show_colcounts = TRUE) |>
    rtables::split_cols_by(var = treat) |>
    tern::analyze_vars(vars = target, var_labels = target_name, na_rm = FALSE) |>
    rtables::build_table(dt)
}

#' Convert a TableTree to a dtlg table
#'
#' [as_dtlg_table()] reformats a TableTree object into a format close to that
#' of dtlg's `data.table`.
#'
#' @param tt A TableTree object. Typically obtained with [tern_summary_table()].
#'
#' @param .label_col Label for stats' column.
#'
#' @returns A data.table.
#'
#' @examples
#' vars <- c('AGE', 'RACE', 'ETHNIC', 'BMRKR1')
#' var_labels <- c("Age (yr)", "Race", "Ethnicity", "Continuous Level Biomarker 1")
#'
#' # Summary statistics table split by ARM with custom labels.
#' (tt <- tern_summary_table(
#'   adsl,
#'   target = vars,
#'   treat = 'ARM',
#'   target_name = var_labels
#' ))
#'
#' # Format as a dtlg table
#' as_dtlg_table(tt)
#'
#' @export
as_dtlg_table <- function(tt, .label_col = "stats") {

  assert_pkg_is_installed("rtables")
  assert_pkg_is_installed("dplyr")

  rtables::as_result_df(
    tt,
    simplify = TRUE,
    keep_label_rows = TRUE,
    data_format = "strings"
  ) |>
    dplyr::rename("{.label_col}" := "label_name") |>
    dplyr::mutate(dplyr::across(is.character, ~ dplyr::if_else(is.na(.x), "", .x))) |>
    data.table::as.data.table()
}

#' Generate Core Safety Tables (CSR Section 14.3.1) using `tern`/`rtables`
#'
#' [tern_AET01_table()] produces a consolidated safety summary table using
#' `rtables` and `tern`. It mirrors the output and interface of
#' [AET01_table()], generating standard adverse event summaries (e.g. death,
#' withdrawal, AESIs) for Clinical Study Reports (CSR) Section 14.3.1.
#'
#' The function returns a single formatted `rtables` table summarising core
#' safety endpoints by treatment arm.
#'
#' @param adsl A subject-level dataset (typically ADaM ADSL).
#'
#' @param adae A dataset of adverse events, preprocessed with AESI flags.
#'
#' @param patient_var A string indicating the subject identifier variable (e.g.,
#'   `"USUBJID"`).
#'
#' @param treat_var A string indicating the treatment arm variable (e.g., `"ARM"`).
#'
#' @param aesi_vars A character vector of binary AESI flags in `adae`.
#'
#' @param aesi_heading Ignored (included for interface compatibility).
#'
#' @param indent Ignored (included for interface compatibility).
#'
#' @return A `TableTree` object from the `rtables` package.
#'
#' @examples
#' tern_AET01_table(
#'   adsl = adsl,
#'   adae = aesi,
#'   patient_var = "USUBJID",
#'   treat_var = "ARM",
#'   aesi_vars = c("FATAL", "SER", "SERWD", "SERDSM", "RELSER",
#'                 "WD", "DSM", "REL", "RELWD", "RELDSM", "SEV")
#' )
#'
#' @export
tern_AET01_table <- function(adsl,
                             adae,
                             patient_var,
                             treat_var,
                             aesi_vars,
                             aesi_heading = "Total number of patients with at least one",
                             indent = "  ") {

  # --- Input validation ---
  stopifnot(is.data.frame(adsl), is.data.frame(adae))

  required_vars <- c(patient_var, treat_var)
  missing_vars_adsl <- setdiff(required_vars, names(adsl))
  missing_vars_adae <- setdiff(required_vars, names(adae))

  if (length(missing_vars_adsl) > 0) {
    stop("The following variables are missing in `adsl`: ", toString(missing_vars_adsl))
  }

  if (length(missing_vars_adae) > 0) {
    stop("The following variables are missing in `adae`: ", toString(missing_vars_adae))
  }

  if (!is.character(aesi_vars) || length(aesi_vars) == 0) {
    stop("`aesi_vars` must be a non-empty character vector.")
  }

  missing_aesi <- setdiff(aesi_vars, names(adae))
  if (length(missing_aesi) > 0) {
    stop("The following `aesi_vars` are not in `adae`: ", toString(missing_aesi))
  }

  # --- ADSL layout: deaths and withdrawals ---
  lyt_adsl <- rtables::basic_table(show_colcounts = TRUE) |>
    rtables::split_cols_by(treat_var) |>
    tern::count_patients_with_event(
      patient_var,
      filters = c("DTHFL" = "Y"),
      denom = "N_col",
      .labels = c(count_fraction = "Total number of deaths"),
      table_names = "death_count"
    ) |>
    tern::count_patients_with_event(
      patient_var,
      filters = c("DCSREAS" = "ADVERSE EVENT"),
      denom = "N_col",
      .labels = c(count_fraction = "Total number of patients withdrawn from study due to an AE"),
      table_names = "ae_withdrawal"
    )

  tbl_adsl <- rtables::build_table(lyt_adsl, df = adsl, alt_counts_df = adsl)

  # --- ADAE layout: AE summary and AESIs ---
  aesi_labels <- sapply(aesi_vars, \(x) label(adae[[x]]) %||% x)

  lyt_adae <- rtables::basic_table(show_colcounts = TRUE) |>
    rtables::split_cols_by(treat_var) |>
    tern::analyze_num_patients(
      vars = patient_var,
      .stats = c("unique", "nonunique"),
      .labels = c(
        unique = "Total number of patients with at least one AE",
        nonunique = "Total number of AEs"
      ),
      .formats = list(unique = tern::format_count_fraction_fixed_dp, nonunique = "xx"),
      show_labels = "hidden"
    ) |>
    tern::count_patients_with_flags(
      patient_var,
      flag_variables = aesi_vars,
      .labels = aesi_labels,
      denom = "N_col",
      var_labels = aesi_heading,
      show_labels = "visible"
    )

  tbl_adae <- rtables::build_table(lyt_adae, df = adae, alt_counts_df = adsl)

  # --- Merge final result with consistent col_info ---
  rtables::col_info(tbl_adsl) <- rtables::col_info(tbl_adae)

  rbind(
    tbl_adae[1:2, ],
    tbl_adsl,
    tbl_adae[3:nrow(tbl_adae), ]
  )
}

#' Generate AET02-style AE summary using `tern` and `rtables`
#'
#' This function builds a System Organ Class (SOC) and Preferred Term (PT)
#' adverse event summary table, following the AET02 CSR format, using the
#' `tern` and `rtables` packages.
#'
#' @param adsl Subject-level dataset.
#' @param adae Adverse event dataset.
#' @param patient Unique subject identifier variable.
#' @param treat Treatment arm variable.
#' @param target Preferred term variable (default: `"AEDECOD"`).
#' @param rows_by Higher-level nesting term (default: `"AEBODSYS"`).
#' @param indent Ignored (included for compatibility).
#'
#' @returns A `TableTree` object with AE summary by SOC/PT.
#'
#' @seealso [AET02_table()]
#'
#' @export
tern_AET02_table <- function(adsl,
                             adae,
                             patient,
                             treat,
                             target = "AEDECOD",
                             rows_by = "AEBODSYS",
                             indent = "  ") {

  stopifnot(is.data.frame(adsl), is.data.frame(adae))
  stopifnot(all(c(patient, treat) %in% names(adsl)))
  stopifnot(all(c(patient, treat, target, rows_by) %in% names(adae)))

  adae <- tern::df_explicit_na(adae, na_level = "Missing")

  lyt <- rtables::basic_table(show_colcounts = TRUE) |>
    rtables::split_cols_by(var = treat) |>
    rtables::split_rows_by(var = rows_by, split_fun = rtables::drop_split_levels, nested = TRUE) |>
    tern::count_occurrences(
      vars = target,
      id = patient,
      denom = "N_col",
      .labels = c(count_fraction = "Patients with AE (n, %)"),
      .formats = c(count_fraction = tern::format_count_fraction_fixed_dp)
    )

  rtables::build_table(lyt, df = adae, alt_counts_df = adsl)
}

test_that("DMT01 Demographics Table can be replicated using tern functions", {
  dmg_vars <- c("AGE", "RACE", "ETHNIC")
  dmg_var_lbls <- c("Age (yr)", "Race", "Ethnicity")

  # Demographics table (DMT01)
  dmt_01 <- tern_summary_table(
    adsl,
    target = dmg_vars,
    treat = "ARM",
    target_name = dmg_var_lbls
  )

  expect_s4_class(dmt_01, "TableTree")
  expect_named(dmt_01@children, dmg_vars)
  expect_named(dmt_01@col_info@subset_exprs, levels(adsl$ARM))
})

#' Calculate descriptive summary statistics for a numeric variable
#'
#' @description
#'
#' [calc_desc()] summarises a numeric variable (`target`) by another (`treat`) and
#' reports summary statistics in clinical trial reporting format. The following
#' statistics are calculated for `target`, per group, i.e. by variable `treat`
#' levels:
#'
#' - `n`: number of observations
#' - `Mean (SD)`: mean and standard deviation of `target`
#' - `Median`: median of `target`
#' - `Min, Max`: minimum and maximum of `target`
#' - `Missing`: number of missing `target` values
#'
#' @param dt A `data.frame` containing, at least, the variables indicated in
#' `target` and `treat`.
#' @param target Target variable passed as a string for which summary
#'   statistics are to be calculated.
#' @param target_name Heading for the target variable as a string. Defaults to
#' `target`.
#' @param treat A string indicating the grouping variable, e.g. the variable
#' specifying the treatment population.
#' @param indent A string to be used as indentation of summary statistics
#' labels. Defaults to four HTML non-breaking spaces (`&nbsp;`).
#' @param pct_dec Decimal places for reported figures.
#' @param inc_missing Toggle for including the "Missing" row:
#' \describe{
#' \item{`TRUE`}{(default) The Missing row is always displayed}
#' \item{`NA`}{The Missing row is only displayed if any missing values are present}
#' \item{`FALSE`}{The Missing row is never included in the table}
#' }
#'
#' @returns A list containing a `data.table` formatted as follows:
#'
#' - First column is named `stats` and contains the target variable name
#' indicated in `target` in the first row. Subsequent rows contain the
#' summarised statistics labels.
#'
#' - Other columns are for the levels of the grouping variable (`treat`).
#'
#' - All columns are of character type.
#'
#' This table is structured for easy integration with Shiny output widgets.
#'
#' @examples
#' # Calculate summary statistics for the age of the subjects in each region.
#' calc_desc(dt = adsl, "AGE", treat = "REGION1")[[1]]
#'
#' # Calculate summary statistics for biomarker 1 in each of the three arms
#' # (`ARM`).
#' calc_desc(dt = adsl, "BMRKR1", treat = "ARM")[[1]]
#'
#' @export
#'
calc_desc <- function(dt,
                      target,
                      target_name = target,
                      treat,
                      indent = nbsp(n = 4L),
                      pct_dec = 1,
                      inc_missing = TRUE) {

  stopifnot(
    is.data.frame(dt),
    is_string(target),
    is_string(target_name),
    is_string(treat),
    is_string(indent),
    is.numeric(pct_dec)
  )

  dt <- maybe_copy_dt(x = dt)

  dt_stats <- dt_summarise(
    dt,
    .by = treat,
    n = as.character(sum(!is.na(target))),
    `Mean (SD)` = format_mean_sd(
      mean = mean(target, na.rm = TRUE),
      sd = stats::sd(target, na.rm = TRUE),
      .digits = pct_dec
    ),
    Median = round(
      stats::median(target, na.rm = TRUE),
      digits = pct_dec
    ),
    `Min, Max` = format_min_max(
      min = min(target, na.rm = TRUE),
      max = max(target, na.rm = TRUE),
      .digits = pct_dec
    ),
    .env = list(target = as.name(target), pct_dec = pct_dec)
  )

  if (isTRUE(inc_missing) || (is.na(inc_missing) && anyNA(dt[[target]]))) {
    dt_missing <- dt_summarise(
      dt,
      .by = treat,
      Missing = as.character(sum(is.na(target))),
      .env = list(target = as.name(target))
    )

    dt_stats <- dt_stats[dt_missing, on = treat]
  }

  dt_stats <- data.table::transpose(dt_stats, keep.names = "stats", make.names = treat)
  header <- c(target_name, rep(list(""), times = ncol(dt_stats) - 1L))
  dt_stats[, `:=`(stats = indent(dt_stats$stats, indentation = indent))]
  dt_stats <- rbind(header, dt_stats)

  list(dt_stats)
}


#' Calculate counts of a categorical variable
#'
#' [calc_counts()] counts observations of a categorical variable (`target`) by
#' another (`treat`) and reports summary statistics in clinical trial reporting
#' format.
#'
#' @param dt A `data.frame` containing, at least, the variables indicated in
#' `target` and `treat`.
#' @param target Target variable passed as a string for which summary
#'   statistics are to be calculated.
#' @param target_name Heading for the target variable as a string. Defaults to
#' `target`.
#' @param treat A string indicating the grouping variable, e.g. the variable
#' specifying the treatment population.
#' @param indent A string to be used as indentation of summary statistics
#' labels. Defaults to four HTML non-breaking spaces (`&nbsp;`).
#' @param .total_dt Separate table from `dt` from which to derive total counts
#' per group.
#' @param pct_dec This argument is ignored, and is only kept for backward
#' compatibility reasons.
#'
#' @returns A list containing a `data.table` formatted as follows:
#'
#' - First column is named `stats` and contains the target variable name
#' indicated in `target` in the first row. Subsequent rows contain the
#' levels of `target`.
#'
#' - Other columns are for the levels of the grouping variable (`treat`).
#'
#' - All columns are of character type.
#'
#' This table is structured for easy integration with Shiny output widgets.
#'
#' @examples
#' calc_counts(dt = adsl, "RACE", treat = "ARM", indent = "  ")[[1]]
#'
#' @export
#'
calc_counts <- function(dt,
                        target,
                        target_name = target,
                        treat,
                        indent = nbsp(n = 4L),
                        .total_dt = NULL,
                        pct_dec = 1) {

  stopifnot(
    is.data.frame(dt),
    is_string(target),
    is_string(target_name),
    is_string(treat),
    is_string(indent),
    is.null(.total_dt) || is.data.frame(.total_dt)
  )

  dt <- maybe_copy_dt(x = dt)
  counts_by_grp <- dt_count(dt = dt,
                            treatment = treat,
                            stats = target)

  if (!is.null(.total_dt)) {
    .total_dt <- maybe_copy_dt(x = .total_dt)
    counts_by_trt <- dt_count(.total_dt, treatment = treat, .name = "total")
    counts_by_grp <- counts_by_grp[counts_by_trt, on = "treatment"]
    counts_by_grp$n <- format_n_pct(
      n = counts_by_grp$n,
      pct = 100 * counts_by_grp$n / counts_by_grp$total,
      .pct_digits = pct_dec
    )
    j <- dot_wrap(c("treatment", "stats", "n"))
    counts_by_grp <- counts_by_grp[, j, env = list(j = j)]
  }

  counts_by_grp <- data.table::dcast(
    counts_by_grp,
    stats::as.formula("stats ~ treatment"),
    value.var = "n",
    fill = 0
  )

  header <- c(target_name, rep(list(""), times = ncol(counts_by_grp) - 1L))
  counts_by_grp[,  `:=`(stats = paste0(indent, as.character(counts_by_grp$stats)))]
  counts_by_grp <- rbind(header, counts_by_grp)

  list(counts_by_grp)
}


#' Calculate summary statistics for a variable
#'
#' [calc_stats()] calculates summary statistics for a variable on groups. This
#' is a generic function; note that it dispatches based on the class of
#' `target` (second argument), not `dt` (first argument).
#'
#' @inheritParams calc_desc
#' @inheritParams calc_counts
#' @param ... Additional arguments passed to other methods
#'
#' @returns A `data.table` of summary statistics. The format depends on the
#' type of the `target` variable:
#'
#' - If the `target` variable is categorical, i.e. type `character`, `factor`
#' or `logical` then the output is that of [calc_counts()].
#'
#' - If the `target` variable is numeric, then the output is that of
#' [calc_desc()].
#'
#' @examples
#' # Calculate summary statistics of a numeric variable, e.g. `AGE`.
#' calc_stats(dt = adsl, "AGE", treat = "ARM")[[1]]
#'
#' # Calculate summary statistics of a categorical variable, e.g. `SEX`.
#' calc_stats(dt = adsl, "SEX", treat = "ARM")[[1]]
#'
#' @export
calc_stats <- function(dt,
                       target,
                       treat,
                       ...,
                       target_name = target,
                       indent = nbsp(n = 4L),
                       .total_dt = NULL,
                       pct_dec = 1) {
  dt <- maybe_copy_dt(x = dt)
  UseMethod("calc_stats", dt[[target]])
}

#' @export
calc_stats.numeric <- function(dt,
                               target,
                               treat,
                               ...,
                               target_name = target,
                               indent = nbsp(n = 4L),
                               pct_dec = 1,
                               inc_missing = TRUE) {
  calc_desc(
    dt,
    target = target,
    treat = treat,
    target_name = target_name,
    indent = indent,
    inc_missing = inc_missing,
    pct_dec = pct_dec
  )
}

#' @export
calc_stats.character <- function(dt,
                                 target,
                                 treat,
                                 ...,
                                 target_name = target,
                                 indent = nbsp(n = 4L),
                                 .total_dt = NULL,
                                 pct_dec = 1) {
  calc_counts(
    dt = dt,
    target = target,
    target_name = target_name,
    treat = treat,
    indent = indent,
    .total_dt = .total_dt,
    pct_dec = pct_dec
  )
}

#' @export
calc_stats.factor <- function(dt,
                              target,
                              treat,
                              ...,
                              target_name = target,
                              indent = nbsp(n = 4L),
                              .total_dt = NULL,
                              pct_dec = 1) {
  calc_counts(
    dt = dt,
    target = target,
    target_name = target_name,
    treat = treat,
    indent = indent,
    .total_dt = .total_dt,
    pct_dec = pct_dec
  )
}

#' @export
calc_stats.logical <- function(dt,
                               target,
                               treat,
                               ...,
                               target_name = target,
                               indent = nbsp(n = 4L),
                               .total_dt = NULL,
                               pct_dec = 1) {
  calc_counts(
    dt = dt,
    target = target,
    target_name = target_name,
    treat = treat,
    indent = indent,
    .total_dt = .total_dt,
    pct_dec = pct_dec
  )
}

#' @export
calc_stats.default <- function(dt, target, ...) {
  target_class <- class(dt[[target]])
  stop("`cal_stats()` does not support dispatch on `target` variables of class ", target_class, ".")
}

#' Retrieve the label of an object
#'
#' [label()] gets the attached label to an object.
#'
#' @param x An R object.
#'
#' @returns The label attribute (string) associated with object passed in `x` or
#'   `NULL` if the label attribute does not exist.
#'
#' @examples
#' label(1)
#' label(with_label(1, "my label"))
#'
#' @seealso [with_label()]
#'
#' @export
label <- function(x) {
  attr(x, "label")
}

#' Add a label attribute to an object
#'
#' @param x An R object.
#' @param label A label provided as a single string.
#'
#' @return `x` labeled by `label`.
#'
#' @examples
#' label(1)
#' label(with_label(1, "my label"))
#'
#' @seealso [label()]
#'
#' @export
with_label <- function(x, label) {

  stopifnot(is_string(label))
  attr(x, "label") <- label
  x
}

#' Create AET02-style AE summary table
#'
#' Summarises adverse events in a format similar to the AET02 table from a CSR,
#' showing total AE counts, patients with AEs, and a breakdown by System Organ
#' Class (SOC) and Preferred Term (PT).
#'
#' @param adsl Subject-level dataset.
#' @param adae Adverse event dataset.
#' @param patient Unique subject identifier variable.
#' @param treat Treatment arm variable.
#' @param target Preferred term variable for grouping (default: `"AEDECOD"`).
#' @param rows_by Higher-level term for nesting (default: `"AEBODSYS"`).
#' @param indent Character or string to indent nested rows (default: 4 non-breaking spaces).
#'
#' @returns A merged data.table containing AE summary.
#'
#' @examples
#' # Create a AET02 table
#' AET02_table(
#'   adsl = adsl,
#'   adae = aesi,
#'   patient = "USUBJID",
#'   treat = "ARM",
#'   target = "AEDECOD",
#'   rows_by = "AEBODSYS",
#'   indent = "  "
#' )
#'
#' @export
AET02_table <- function(adsl,
                        adae,
                        patient,
                        treat,
                        target = "AEDECOD",
                        rows_by = "AEBODSYS",
                        indent = nbsp(n = 4L)) {

  stopifnot(is.data.frame(adsl), is.data.frame(adae))
  stopifnot(all(c(patient, treat, target, rows_by) %in% colnames(adae)))
  stopifnot(patient %in% colnames(adsl))

  patients_with_ae_table <- event_count(
    dt = adae,
    patient = patient,
    treat = treat,
    label = "Total number of patients with at least one AE",
    .total_dt = adsl
  )

  total_ae_events_table <- total_events(
    dt = adae,
    treat = treat,
    label = "Total number of AEs"
  )

  ae_by_soc_pt_table <- list(
    event_count_by(
      dt = adae,
      patient = patient,
      treat = treat,
      target = target,
      rows_by = rows_by,
      .total_dt = adsl,
      indent = indent
    )
  )

  merge_table_lists(list(
    patients_with_ae_table,
    total_ae_events_table,
    ae_by_soc_pt_table
  ))
}

#' Merge a list of list-wrapped data.tables into one data.table
#'
#' This function is typically used to combine multiple reporting tables,
#' each produced by `event_count()`, `total_events()`, or
#' `multi_event_true()`, into a single summary table. These intermediate tables
#' are often returned as one-element lists containing a `data.table`.
#'
#' This helper unwraps and merges them, row-wise, to produce a
#' consolidated safety report table --- commonly used in clinical study reports
#' or data monitoring reviews.
#'
#' @param dt_l A list of one-element lists, where each element is a list
#'   containing a single `data.table`.
#'
#' @returns A single merged `data.table`, row-bound from all input tables.
#'
#' @examples
#' # Count deaths by treatment arm
#' death_table <- event_count(
#'   adsl,
#'   patient = "USUBJID",
#'   treat = "ARM",
#'   label = "Total number of deaths",
#'   .filters = "DTHFL == 'Y'"
#' )
#'
#' # Count study withdrawals due to adverse events
#' withdrawal_table <- event_count(
#'   adsl,
#'   patient = "USUBJID",
#'   treat = "ARM",
#'   label = "Total number of patients withdrawn from study due to an AE",
#'   .filters = "DCSREAS == 'ADVERSE EVENT'"
#' )
#'
#' # Count patients with at least one adverse event
#' patients_with_ae_table <- event_count(
#'   adae,
#'   patient = "USUBJID",
#'   treat = "ARM",
#'   label = "Total number of patients with at least one AE"
#' )
#'
#' # Count total number of adverse events (not patients)
#' total_ae_events_table <- total_events(
#'   dt = adae,
#'   treat = "ARM",
#'   label = "Total number of AEs"
#' )
#'
#' # Summarise AESIs (e.g., serious, related, severe, etc.)
#' aesi_vars <- c("FATAL", "SER", "SERWD", "SERDSM", "RELSER",
#'                "WD", "DSM", "REL", "RELWD", "RELDSM", "SEV")
#'
#' aesi_table <- multi_event_true(
#'   dt = aesi,
#'   event_vars = aesi_vars,
#'   patient = "USUBJID",
#'   treat = "ARM",
#'   heading = "Total number of patients with at least one",
#'   .total_dt = adsl,
#'   indent = "  "
#' )
#'
#' # Combine all safety tables into a single summary table
#' safety_summary <- merge_table_lists(list(
#'   patients_with_ae_table,
#'   total_ae_events_table,
#'   death_table,
#'   withdrawal_table,
#'   aesi_table
#' ))
#'
#' safety_summary
#'
#' @export
merge_table_lists <- function(dt_l) {
  dt_l <- lapply(dt_l, `[[`, 1)
  data.table::rbindlist(dt_l, use.names = TRUE)
}

#' Print a `dtlg` table
#'
#' A convenience wrapper around [print()] for printing `dtlg` tables with
#' consistent formatting options.
#'
#' @param dt A `dtlg` table, typically a `data.frame` or `data.table`.
#' @inheritParams data.table::print.data.table
#' @param justify String. Column alignment; one of `"left"`, `"right"`,
#'   `"centre"`, or `"none"`. Defaults to `"left"`.
#'
#' @returns Invisibly returns the printed object.
#' @export
#'
#' @examples
#' calc_stats(dt = adsl, "AGE", treat = "ARM", indent = "  ")[[1]] |>
#'   print_dtlg()
#'
print_dtlg <- function(dt,
                       row.names = FALSE,
                       trunc.cols = TRUE,
                       class = FALSE,
                       nrows = Inf,
                       justify = "left") {
  print(
    data.table::as.data.table(dt),
    row.names = row.names,
    trunc.cols = trunc.cols,
    class = class,
    nrows = nrows,
    justify = justify
  )
}

reorder_cols <- function(dt, cols, before_cols = "stats", skip_absent = TRUE) {

  cols %||% return(dt)

  all_cols <- colnames(dt)
  cols_missing <- sort(setdiff(all_cols, cols))
  new_cols <- unique(c(before_cols, cols, cols_missing))
  data.table::setcolorder(dt, new_cols, skip_absent = skip_absent)[]
}

#' Summary Table
#'
#' [summary_table()] summarises clinical variables into a report table using
#' `data.table` as backend.
#'
#' @inheritParams calc_stats
#' @param treat_order Customise the column order of the output table.
#' @param skip_absent Whether to ignore variables passed in `treat_order` that
#'   are absent from `dt`. Default is `TRUE`; `FALSE` will throw an error in
#'   case there are missing variables.
#' @param inc_missing If any of the `target` variables are numeric, then used
#' as a toggle to determine whether or not "Missing" appears in the summary stats:
#' \describe{
#' \item{`TRUE`}{(default) The Missing row is always displayed}
#' \item{`NA`}{The Missing row is only displayed if any missing values are present}
#' \item{`FALSE`}{The Missing row is never included in the table}
#' }
#'
#' @returns A `data.table` of summary statistics. The format depends on the
#' type of the `target` variable:
#'
#' - If the `target` variable is categorical, i.e. type `character`, `factor`
#' or `logical` then the output is that of [calc_counts()].
#'
#' - If the `target` variable is numeric, then the output is that of
#' [calc_desc()].
#'
#' @seealso [tern_summary_table()]
#'
#' @examples
#' dmg_vars <- c("AGE", "RACE", "ETHNIC")
#' dmg_var_lbls <- c("Age (yr)", "Race", "Ethnicity")
#'
#' # Demographics table (DMT01)
#' summary_table(
#'   adsl,
#'   target = dmg_vars,
#'   treat = 'ARM',
#'   target_name = dmg_var_lbls
#' )
#'
#' # Demographics table (DMT01) with continuous variable (e.g., BMRKR1)
#' summary_table(
#'   adsl,
#'   target = c(dmg_vars, "BMRKR1"),
#'   treat = 'ARM',
#'   target_name = c(dmg_var_lbls, "Biomarker 1")
#' )
#'
#' @export
#'
summary_table <- function(dt,
                          target,
                          treat,
                          target_name = target,
                          indent = nbsp(n = 4L),
                          .total_dt = dt,
                          pct_dec = 1,
                          treat_order = NULL,
                          skip_absent = TRUE,
                          inc_missing = TRUE) {

  stopifnot(length(target) >= length(target_name), length(target) >= length(indent))
  vct_args <- vctrs::vec_recycle_common(target = target, target_name = target_name, indent = indent)
  scl_args <- list( # nolint: object_usage_linter
    dt = dt,
    treat = treat,
    .total_dt = .total_dt,
    pct_dec = pct_dec,
    inc_missing = inc_missing
  )

  summaries <- with(
    vct_args,
    expr = mapply(
      FUN = calc_stats,
      target = target,
      target_name = target_name,
      indent = indent,
      MoreArgs = scl_args
    )
  )

  summaries |>
    data.table::rbindlist(use.names = TRUE) |>
    reorder_cols(cols = treat_order, skip_absent = skip_absent)
}

#' Create a summary table using multiple rows for grouping on one target column
#'
#' @inheritParams summary_table
#' @param rows_by string, grouping variable to split events by.
#' @param sep character string to separate the terms
#'
#' @returns The same output as [summary_table()] except that folded by variables
#' indicated in `rows_by`.
#' @export
#'
#' @examples
#' summary_table_by(
#'   adlb,
#'   target = "AVAL",
#'   treat = "ARM",
#'   rows_by = c("PARAM","AVISIT")
#' )
summary_table_by <- function(dt,
                             target,
                             treat,
                             rows_by,
                             indent = nbsp(n = 4L),
                             .total_dt = dt,
                             pct_dec = 1,
                             treat_order = NULL,
                             skip_absent = TRUE,
                             sep = ".") {
  dt <- maybe_copy_dt(x = dt)

  dt <- split(droplevels(dt), by = rows_by, drop = TRUE, sorted = TRUE)
  label <- names(dt)

  if (length(rows_by) > 1) {
    label <- strsplit(label, ".", fixed = TRUE)
    heading_full <- lapply(X = label, function(x) {
      x[1]
    })
    heading <- unique(heading_full)
    label <- lapply(label, \(x) paste(x[-1L], collapse = sep))
    label <- paste0(indent, label)
    indent <- paste0(indent, indent)
  }

  summary_split <- mapply(
    calc_stats,
    dt = dt,
    target = target,
    target_name = label,
    treat = treat,
    indent = indent,
    pct_dec = pct_dec
  )

  if (length(rows_by) > 1) {
    x <- 0
    for (i in seq_along(heading)) {
      y = sum(heading_full %in% heading[i])
      summary_split <- append(
        summary_split,
        list(data.table::data.table(stats = heading[[i]])),
        after = x
      )
      x <- x + y + 1
    }
  }

  summary_split |>
    data.table::rbindlist(use.names = TRUE, fill = TRUE) |>
    reorder_cols(cols = treat_order, skip_absent = skip_absent) |>
    list()
}

#' Create a summary table using multiple rows for grouping on two target column
#' ideal for creating change from baseline tables
#'
#' @param dt table to perform function on
#' @param target vector of column names desired to obtain information on
#' @param treat string of treatment variable used for splitting / grouping data
#' @param rows_by string, grouping variable to split events by.
#' @param indent indent to be used for display and formatting purposes
#' @param .total_dt optional table for total counts to be derived
#' @param pct_dec decimal places for percentages
#' @param treat_order customise the column order of output table
#' @param skip_absent Logical, default `TRUE`. Passed to
#'   `data.table::setcolorder`, if `treat_order` includes columns not present in
#'   `dt`:
#' - `TRUE` will silently ignore them
#' - `FALSE` will throw an error
#' @param sep character string to separate the terms
#'
#' @return data.table
#' @export
#'
#' @examples
#' adlb <- random.cdisc.data::cadlb |>
#'   dplyr::filter(AVISIT != "SCREENING")
#'
#' labs <- summary_table_by_targets(
#'   dt = adlb,
#'   target = c("AVAL", "CHG"),
#'   treat = "ARM",
#'   rows_by = c("PARAM", "AVISIT"),
#'   indent = "  "
#' )
summary_table_by_targets <- function(dt,
                                     target,
                                     treat,
                                     rows_by,
                                     indent = nbsp(n = 4L),
                                     .total_dt = NULL,
                                     pct_dec = 1,
                                     treat_order = NULL,
                                     skip_absent = TRUE,
                                     sep = ".") {
  .total_dt <- maybe_copy_dt(x = .total_dt)

  summary_tables <- mapply(
    summary_table_by,
    target = target,
    MoreArgs = list(
      dt = dt,
      treat = treat,
      rows_by = rows_by,
      indent = indent,
      .total_dt = .total_dt,
      pct_dec = pct_dec,
      treat_order = treat_order,
      skip_absent = skip_absent,
      sep = sep
    )
  )

  for (i in seq_along(summary_tables)) {
    data.table::setnames(
      summary_tables[[i]],
      old = -1L,
      new = paste(names(summary_tables[[i]])[-1L], target[i], sep = sep)
    )
  }

  data.table::cbindlist(
    c(
      list(summary_tables[[1]][, 1, with = FALSE]),
      lapply(
        seq(2, ncol(summary_tables[[1]])),
        \(i) data.table::cbindlist(lapply(summary_tables, \(dat) dat[, i, with = FALSE]))
      )
    )
  )
}

#
# Setup
#
adlb_arm_levels <- levels(adlb$ARM)

#### summary_table ####
test_that("`summary_table()` respects order of levels of `treat` in the output", {
  observed <- summary_table(dt = adlb, target = "AVAL", treat = "ARM")
  treat_cols <- colnames(observed)[-1L]

  expect_equal(object = treat_cols, expected = adlb_arm_levels)
})

test_that("`summary_table()`: `treat_order` works", {
  observed <- summary_table(
    dt = adlb,
    target = "AVAL",
    treat = "ARM",
    treat_order = rev(adlb_arm_levels)
  )
  treat_cols <- colnames(observed)[-1L]

  expect_equal(object = treat_cols, expected = rev(adlb_arm_levels))
})

test_that("`summary_table()`: `treat_order` plays well with `skip_absent`", {
  observed <- summary_table(
    dt = adlb,
    target = "AVAL",
    treat = "ARM",
    treat_order = c(rev(adlb_arm_levels), "PHONY"),
    skip_absent = TRUE
  )
  treat_cols <- colnames(observed)[-1L]
  expect_equal(object = treat_cols, expected = rev(adlb_arm_levels))

  expect_error(
    summary_table(
      dt = adlb,
      target = "AVAL",
      treat = "ARM",
      treat_order = c(rev(adlb_arm_levels), "PHONY"),
      skip_absent = FALSE
    )
  )
})

test_that("`summary_table()` tolerates 'stats' in `treat_order`", {

  #
  # `treat_order = "stats"` # nolint: commented_code_linter
  #
  wo_stats <- summary_table(
    dt = adlb,
    target = "AVAL",
    treat = "ARM",
    treat_order = NULL
  )

  wt_stats <- summary_table(
    dt = adlb,
    target = "AVAL",
    treat = "ARM",
    treat_order = "stats"
  )

  testthat::expect_equal(object = wt_stats, expected = wo_stats)

  #
  # `treat_order = rep("stats", 3L)` # nolint: commented_code_linter
  #
  wo_stats <- summary_table(
    dt = adlb,
    target = "AVAL",
    treat = "ARM",
    treat_order = NULL
  )

  wt_stats <- summary_table(
    dt = adlb,
    target = "AVAL",
    treat = "ARM",
    treat_order = rep("stats", 3L)
  )

  testthat::expect_equal(object = wt_stats, expected = wo_stats)

  #
  # `treat_order = c("stats", adlb_arm_levels[1])` # nolint: commented_code_linter
  #
  wo_stats <- summary_table(
    dt = adlb,
    target = "AVAL",
    treat = "ARM",
    treat_order = NULL
  )

  wt_stats <- summary_table(
    dt = adlb,
    target = "AVAL",
    treat = "ARM",
    treat_order = c("stats", adlb_arm_levels[1])
  )

  testthat::expect_equal(object = wt_stats, expected = wo_stats)

  #
  # `treat_order = c("stats", adlb_arm_levels[1])` # nolint: commented_code_linter
  #
  wo_stats <- summary_table(
    dt = adlb,
    target = "AVAL",
    treat = "ARM",
    treat_order = NULL
  )

  wt_stats <- summary_table(
    dt = adlb,
    target = "AVAL",
    treat = "ARM",
    treat_order = c(adlb_arm_levels[1], "stats")
  )

  testthat::expect_equal(object = wt_stats, expected = wo_stats)
})

test_that("`summary_table()`: `pct_dec` works", {

  pct_dec <- 0L
  observed <- summary_table(
    dt = adlb,
    target = "AVAL",
    treat = "ARM",
    pct_dec = pct_dec
  )

  n_mantissa <- tools::file_ext(observed[2L, -1L])
  median_mantissa <- tools::file_ext(observed[4L, -1L])

  # No mantissa is expected for integer variables
  expect_equal(object = 0L, expected = unique(nchar(n_mantissa)))
  # Mantissa length should match `pct_dec`.
  expect_equal(object = pct_dec, expected = unique(nchar(median_mantissa)))

  pct_dec <- 1L
  observed <- summary_table(
    dt = adlb,
    target = "AVAL",
    treat = "ARM",
    pct_dec = pct_dec
  )

  n_mantissa <- tools::file_ext(observed[2L, -1L])
  median_mantissa <- tools::file_ext(observed[4L, -1L])

  # No mantissa is expected for integer variables
  expect_equal(object = 0L, expected = unique(nchar(n_mantissa)))
  # Mantissa length should match `pct_dec`.
  expect_equal(object = pct_dec, expected = unique(nchar(median_mantissa)))

  pct_dec <- 2L
  observed <- summary_table(
    dt = adlb,
    target = "AVAL",
    treat = "ARM",
    pct_dec = pct_dec
  )

  n_mantissa <- tools::file_ext(observed[2L, -1L])
  median_mantissa <- tools::file_ext(observed[4L, -1L])

  # No mantissa is expected for integer variables
  expect_equal(object = 0L, expected = unique(nchar(n_mantissa)))
  # Mantissa length should match `pct_dec`.
  expect_equal(object = pct_dec, expected = unique(nchar(median_mantissa)))

  pct_dec <- 10L
  observed <- summary_table(
    dt = adlb,
    target = "AVAL",
    treat = "ARM",
    pct_dec = pct_dec
  )

  n_mantissa <- tools::file_ext(observed[2L, -1L])
  median_mantissa <- tools::file_ext(observed[4L, -1L])

  # No mantissa is expected for integer variables
  expect_equal(object = 0L, expected = unique(nchar(n_mantissa)))
  # Mantissa length should match `pct_dec`.
  expect_equal(object = pct_dec, expected = unique(nchar(median_mantissa)))
})

#### summary_table_by_targets ####
test_that("summary_table_by_targets: Using custom separator does not affect data", {
  summary_dot <- suppressWarnings(
    summary_table_by_targets(
      dt = adlb,
      target = c("AVAL", "CHG"),
      treat = "ARM",
      rows_by = c("PARAM", "AVISIT"),
      indent = "  "
    )
  )

  summary_dash <- suppressWarnings(
    summary_table_by_targets(
      dt = adlb,
      target = c("AVAL", "CHG"),
      treat = "ARM",
      rows_by = c("PARAM", "AVISIT"),
      indent = "  ",
      sep = " - "
    )
  )

  treat_aval_cols <- as.vector(t(outer(adlb_arm_levels, c("AVAL", "CHG"), paste, sep = " - ")))
  expect_named(summary_dash, c("stats", treat_aval_cols))
  expect_identical(unname(summary_dash), unname(summary_dot))
})

test_that("summary_table_by_targets: Able to use more than 2 target variables", {
  target_vars <- c("AVAL", "CHG", "ANRLO", "ANRHI")

  summary_dt <- suppressWarnings(
    summary_table_by_targets(
      dt = adlb,
      target = target_vars,
      treat = "ARM",
      rows_by = c("PARAM", "AVISIT"),
      indent = "  "
    )
  )

  treat_aval_cols <- as.vector(t(outer(adlb_arm_levels, target_vars, paste, sep = ".")))
  expect_named(summary_dt, c("stats", treat_aval_cols))
})

#' Regex for number sign
#'
#' @param sign Whether the number can have a any sign (`"any"`), or be a
#' positive or negative number.
#'
#' @returns A regex as a string.
#'
#' @examples
#' sign_fmt("any")
#'
#' # Strictly positive
#' sign_fmt("any")
#'
#' # Strictly negative
#' sign_fmt("negative")
#'
#' @noRd
#'
sign_fmt <- function(sign = c("any", "positive", "negative")) {
  sign <- match.arg(sign)
  fmt <- c(any = "[-+]?", positive = "[+]?", negative = "-")
  unname(fmt[sign])
}

#' Regex for fractional part of a number
#'
#' @param n Number of decimal places.
#'
#' @returns A regex as a string.
#'
#' @examples
#' fractional_fmt(0L)
#'
#' # One decimal place
#' fractional_fmt(1L)
#'
#' @noRd
#'
fractional_fmt <- function(n = 1L) {
  stopifnot(is.integer(n), n >= 0L)

  if (identical(n, 0L)) {
    ""
  } else {
    sprintf("\\.\\d{%d}", n)
  }
}

#' Regex for a whole number
#'
#' @param sign Whether the number can have a any sign (`"any"`), or be a
#' positive or negative number.
#'
#' @returns A regex as a string.
#'
#' @examples
#' # Positive or negative whole number
#' int_fmt()
#'
#' # Strictly positive number
#' int_fmt("positive")
#'
#' # Strictly negative number
#' int_fmt("negative")
#'
#' @noRd
#'
int_fmt <- function(sign = c("any", "positive", "negative")) {
  sign <- match.arg(sign)
  sprintf("%s\\d+", sign_fmt(sign = sign))
}

#' Regex for a number with fractional part
#'
#' @param n Number of decimal places.
#'
#' @param sign Whether the number can have a any sign (`"any"`), or be a
#' positive or negative number.
#'
#' @returns A regex as a string.
#'
#' @examples
#' # Positive or negative whole number
#' dbl_fmt(0L)
#'
#' # Strictly positive number with one decimal place
#' dbl_fmt(1L, "positive")
#'
#' # Strictly negative number with two decimal places
#' dbl_fmt(2L, "negative")
#'
#' @noRd
#'
dbl_fmt <- function(n = 1L, sign = c("any", "positive", "negative")) {
  stopifnot(is.integer(n), n >= 0L)
  sign <- match.arg(sign)
  sprintf("%s%s", int_fmt(sign = sign), fractional_fmt(n = n))
}

#' Regex for count and percentage (`<n> (<pct>%)`)
#'
#' @param n Number of decimal places for the percentage number.
#'
#' @param support_zero Whether the regex should also support the pattern of a
#'   single zero (`"0"`), i.e. with the percentage omitted. Default is `TRUE`.
#'
#' @returns A regex as a string.
#'
#' @examples
#' # Count and percentage as integer numbers
#' n_pct_fmt(n = 0L, support_zero = FALSE)
#'
#' # Percentage with one decimal place
#' n_pct_fmt(n = 1L, support_zero = FALSE)
#'
#' # Also accept a pattern that is a single `"0"`
#' n_pct_fmt(n = 1L, support_zero = TRUE)
#'
#' @noRd
#'
n_pct_fmt <- function(n = 0L, support_zero = TRUE) {
  zero_fmt <- `if`(support_zero, "^0$|", "")
  sprintf("%s^%s \\(%s%%\\)$",
          zero_fmt,
          int_fmt("positive"),
          dbl_fmt(n = n, "positive"))
}

#' Regex for mean and standard deviation (`<mean> (<sd>)`)
#'
#' @param n Number of decimal places for the mean and standard deviation,
#' defaults to 1.
#'
#' @returns A regex as a string matching the pattern `<mean> (<sd>)`. The
#' standard deviation can't be negative, but may be missing.
#'
#' @examples
#' # Mean and standard deviation as whole numbers
#' mean_sd_fmt(n = 0L)
#'
#' # Mean and standard deviation with one decimal place
#' #' mean_sd_fmt(n = 1L)
#'
#' @noRd
#'
mean_sd_fmt <- function(n = 1L) {
  mean_fmt <- dbl_fmt(n = n)
  sd_fmt <- dbl_fmt(n = n, sign = "positive")
  ws_fmt <- " "
  sprintf("^%s(%s\\(%s\\))?$", mean_fmt, ws_fmt, sd_fmt)
}

min_max_fmt <- function(n = 1L) {
  sprintf("^%s, %s$", dbl_fmt(n = n), dbl_fmt(n = n))
}

detect_fmt <- function(x, fmt) {
  grepl(pattern = fmt, x = x)
}

#' Format count(s) and percentage(s) (`n (pct%)`)
#'
#' [format_n_pct()] formats counts (`n`) and respective percentages (`pct`) as
#' `"n (pct%)"`.
#'
#' @param n An integer vector of counts. Length must match that of `pct`.
#' @param pct A numeric vector of percentages. Length must match that of `n`.
#' @param .pct_digits Number of decimal places to format percentage values.
#'   Defaults to `1`.
#'
#' @returns A character vector of strings following the format `"n (pct%)"`,
#' except if `n` is zero, then the format is simply `"0"`.
#'
#' @examples
#' # Simple cases.
#' format_n_pct(n = 25, pct = 18.66)
#' format_n_pct(n = 25, pct = 18.66, .pct_digits = 2)
#'
#' # If both `n` and `pct` are zero then the format is special, i.e. simply "0".
#' format_n_pct(n = 0, pct = 0)
#'
#' # `format_n_pct()` is vectorised over `n` and `pct` but their length must
#' # match.
#' format_n_pct(n = c(20, 50), pct = c(10, 25))
#'
#' # Missing values result in empty strings in the output.
#' format_n_pct(n = c(20, NA), pct = c(10, 25))
#' format_n_pct(n = c(20, 50), pct = c(10, NA))
#'
#' @keywords internal
#' @export
format_n_pct <- function(n, pct, .pct_digits = 1L) {

  stopifnot(
    is.numeric(n),
    is.numeric(pct),
    is_scalar_number(.pct_digits),
    .pct_digits >= 0L,
    identical(length(n), length(pct))
  )

  is_zero <- n == 0L & is_zero(pct)

  n_fmt <- "%d"
  pct_fmt <- paste0("(%2.", .pct_digits, "f%%)")

  n_str <- sprintf(fmt = n_fmt, n)
  pct_str <- sprintf(fmt = pct_fmt, pct)
  pct_str <- ifelse(is_zero, "", pct_str)

  ws_str <- rep_len(" ", length.out = length(n))
  ws_str <- ifelse(is_zero, "", ws_str)

  n_str[is.na(n)] <- ""
  pct_str[is.na(pct)] <- ""
  ws_str[!(!is.na(n) & !is.na(pct))] <- ""

  sprintf(fmt = "%s%s%s", n_str, ws_str, pct_str)
}

format_mean_sd <- function(mean, sd, .digits = 1L) {

  stopifnot(
    is.numeric(mean),
    is.numeric(sd),
    is_scalar_number(.digits),
    .digits >= 0L,
    identical(length(mean), length(sd))
  )

  mean_fmt <- paste0("%.", .digits, "f")
  sd_fmt <- paste0("(%.", .digits, "f)")

  mean_str <- sprintf(fmt = mean_fmt, mean)
  sd_str <- sprintf(fmt = sd_fmt, sd)
  ws_str <- rep_len(" ", length.out = length(mean))

  mean_str[is.na(mean)] <- ""
  sd_str[is.na(sd)] <- ""
  ws_str[!(!is.na(mean) & !is.na(sd))] <- ""

  sprintf(fmt = "%s%s%s", mean_str, ws_str, sd_str)
}

format_min_max <- function(min, max, .digits = 1L) {

  stopifnot(
    is.numeric(max),
    is.numeric(min),
    is_scalar_number(.digits),
    .digits >= 0L,
    identical(length(max), length(min))
  )

  fmt <- paste0("%.", .digits, "f")

  min_str <- sprintf(fmt = fmt, min)
  max_str <- sprintf(fmt = fmt, max)
  ws_str <- rep_len(", ", length.out = length(max))

  max_str[is.na(max)] <- ""
  min_str[is.na(min)] <- ""
  ws_str[!(!is.na(max) & !is.na(min))] <- ""

  sprintf(fmt = "%s%s%s", min_str, ws_str, max_str)
}

adsl <- data.frame(
  USUBJID = sprintf("ID-%03d", 1:6),
  ARM = c("A", "A", "B", "B", "C", "C"),
  AGE = c(30, 35, 40, 42, 25, NA),
  stringsAsFactors = FALSE
)

test_that("calc_desc() returns expected structure and types", {
  tbl <- calc_desc(adsl, target = "AGE", treat = "ARM", indent = "")[[1]]

  expect_s3_class(tbl, "data.table")
  expect_equal(colnames(tbl), c("stats", "A", "B", "C"))
  expect_equal(tbl$stats[1], "AGE")
  expect_type(tbl$A, "character")
  expect_type(tbl$B, "character")
  expect_type(tbl$C, "character")
})

test_that("calc_desc() reports five statistics per group", {
  tbl <- calc_desc(adsl, target = "AGE", treat = "ARM", indent = "")[[1]]
  expect_equal(nrow(tbl), 1 + 5)  # 1 for header + 5 stats
  expect_equal(tbl$stats[-1], c("n", "Mean (SD)", "Median", "Min, Max", "Missing"))
})

test_that("calc_desc() calculates correct n and Missing", {
  tbl <- calc_desc(adsl, target = "AGE", treat = "ARM", indent = "")[[1]]

  expect_equal(tbl[stats == "n", A], "2")
  expect_equal(tbl[stats == "n", C], "1")

  expect_equal(tbl[stats == "Missing", A], "0")
  expect_equal(tbl[stats == "Missing", C], "1")
})

test_that("calc_desc() formats summary strings correctly", {
  tbl <- calc_desc(adsl, target = "AGE", treat = "ARM", indent = "")[[1]]
  mean_sd_values <- tbl[stats == "Mean (SD)", -1]
  min_max_values <- tbl[stats == "Min, Max", -1]

  expect_true(all(detect_fmt(unlist(mean_sd_values), mean_sd_fmt(n = 1L))))
  expect_true(all(detect_fmt(unlist(min_max_values), min_max_fmt(n = 1L))))
})

test_that("calc_desc() only displays Missing when required if inc_missing is NA", {
  tbl <- calc_desc(adsl, target = "AGE", treat = "ARM", indent = "", inc_missing = NA)[[1]]
  expect_true("Missing" %in% tbl$stats)

  tbl <- calc_desc(adsl[1:5], target = "AGE", treat = "ARM", indent = "", inc_missing = NA)[[1]]
  expect_false("Missing" %in% tbl$stats)
})

test_that("calc_desc() does not display Missing if inc_missing is FALSE", {
  tbl <- calc_desc(adsl, target = "AGE", treat = "ARM", indent = "", inc_missing = FALSE)[[1]]
  expect_false("Missing" %in% tbl$stats)
})

test_that("dbl_fmt() returns correct regex for valid integer input", {
  expect_equal(dbl_fmt(0L), "[-+]?\\d+")
  expect_equal(dbl_fmt(1L), "[-+]?\\d+\\.\\d{1}")
  expect_equal(dbl_fmt(2L), "[-+]?\\d+\\.\\d{2}")
  expect_equal(dbl_fmt(5L), "[-+]?\\d+\\.\\d{5}")
})

test_that("dbl_fmt() errors on non-integer or negative input", {
  expect_error(dbl_fmt("2"))
  expect_error(dbl_fmt(2.0))           # not strictly integer class
  expect_error(dbl_fmt(-1L))           # negative
  expect_error(dbl_fmt(NA_integer_))   # NA
})

test_that("int_fmt() is equivalent to dbl_fmt(0L)", {
  expect_identical(int_fmt(), dbl_fmt(0L))
  expect_equal(int_fmt(), "[-+]?\\d+")
})

test_that("n_pct_fmt() returns correct pattern for support_zero = TRUE", {
  expect_equal(
    n_pct_fmt(0L),
    "^0$|^[+]?\\d+ \\([+]?\\d+%\\)$"
  )

  expect_equal(
    n_pct_fmt(1L),
    "^0$|^[+]?\\d+ \\([+]?\\d+\\.\\d{1}%\\)$"
  )

  expect_equal(
    n_pct_fmt(2L),
    "^0$|^[+]?\\d+ \\([+]?\\d+\\.\\d{2}%\\)$"
  )
})

test_that("n_pct_fmt() returns correct pattern for support_zero = FALSE", {
  expect_equal(
    n_pct_fmt(0L, support_zero = FALSE),
    "^[+]?\\d+ \\([+]?\\d+%\\)$"
  )

  expect_equal(
    n_pct_fmt(2L, support_zero = FALSE),
    "^[+]?\\d+ \\([+]?\\d+\\.\\d{2}%\\)$"
  )
})

test_that("n_pct_fmt() errors on invalid input", {
  expect_error(n_pct_fmt(-1L))
  expect_error(n_pct_fmt("1"))
  expect_error(n_pct_fmt(1.5))
  expect_error(n_pct_fmt(NA_integer_))
})

test_that("detect_fmt() correctly detects valid formatted strings", {
  expect_true(detect_fmt("0", fmt = n_pct_fmt()))
  expect_true(detect_fmt("25 (18%)", n_pct_fmt()))
  expect_true(detect_fmt("25 (18.5%)", n_pct_fmt(1L)))
  expect_true(detect_fmt("99 (100.00%)", n_pct_fmt(2L)))
})

test_that("detect_fmt() returns FALSE for invalid formats", {
  expect_false(detect_fmt("25", n_pct_fmt()))             # missing pct
  expect_false(detect_fmt("25 (18.0%)", n_pct_fmt()))     # should be int
  expect_false(detect_fmt("25 (18%)", n_pct_fmt(1L)))     # missing decimal
  expect_false(detect_fmt("25 (18.500%)", n_pct_fmt(2L))) # too many decimals
  expect_false(detect_fmt("25 18%", n_pct_fmt(1L)))       # no parentheses
  expect_false(detect_fmt("n (x%)", n_pct_fmt(1L)))       # malformed
})

test_that("detect_fmt() is vectorised", {
  x <- c("0", "25 (18%)", "25 (18.5%)", "wrong")
  expect_equal(
    detect_fmt(x,  n_pct_fmt(1L)),
    c(TRUE, FALSE, TRUE, FALSE)
  )
})

test_that("detect_fmt() works with mean_sd_fmt()", {
  fmt <- mean_sd_fmt(1L)

  # Valid cases
  expect_true(detect_fmt("12.3 (4.5)", fmt))
  expect_true(detect_fmt("-12.3 (0.0)", fmt))
  expect_true(detect_fmt("+12.3 (10.0)", fmt))
  expect_true(detect_fmt("12.3", fmt))  # SD missing is allowed

  # Invalid cases
  expect_false(detect_fmt("12.3 (4.56)", fmt))   # too many decimals
  expect_false(detect_fmt("12.3 (4)", fmt))      # missing decimal digits
  expect_false(detect_fmt("12.3 ()", fmt))       # empty parentheses
  expect_false(detect_fmt("12.3 (4.5", fmt))     # missing closing parenthesis
  expect_false(detect_fmt("12.3 4.5", fmt))      # missing parentheses
  expect_false(detect_fmt("abc (4.5)", fmt))     # non-numeric
})

test_that("detect_fmt() works with min_max_fmt()", {
  fmt <- min_max_fmt(1L)

  # Valid cases
  expect_true(detect_fmt("1.0, 5.0", fmt))
  expect_true(detect_fmt("-2.3, -1.2", fmt))
  expect_true(detect_fmt("+2.3, +3.4", fmt))

  # Invalid cases
  expect_false(detect_fmt("1.00, 5.0", fmt))  # too many decimals
  expect_false(detect_fmt("1.0 5.0", fmt))    # missing comma
  expect_false(detect_fmt("1.0,5.0", fmt))    # missing space after comma
  expect_false(detect_fmt("1.0, ", fmt))      # missing max
  expect_false(detect_fmt("a, b", fmt))       # non-numeric
})

adsl <- data.frame(
  USUBJID = sprintf("ID-%03d", 1:6),
  ARM = c("A", "A", "B", "B", "C", "C"),
  RACE = c("White", "Black", "White", "White", "Asian", "Asian"),
  FLAG = c(TRUE, FALSE, TRUE, TRUE, NA, FALSE),
  stringsAsFactors = FALSE
)

adsl_total <- data.frame(
  USUBJID = sprintf("ID-%03d", 7:18),
  ARM = rep(c("A", "B", "C"), each = 4),
  RACE = rep(c("White", "Black", "Asian", "Other"), times = 3),
  stringsAsFactors = FALSE
)


test_that("calc_counts() returns a list with a single data.table", {
  out <- calc_counts(dt = adsl, target = "RACE", treat = "ARM", indent = "  ")
  expect_type(out, "list")
  expect_length(out, 1L)
  expect_s3_class(out[[1]], "data.table")
})

test_that("calc_counts() correctly counts and casts values", {
  tbl <- calc_counts(dt = adsl, target = "RACE", treat = "ARM", indent = "")[[1]]
  expect_equal(colnames(tbl), c("stats", "A", "B", "C"))
  expect_equal(tbl[2L, C], "2")  # Asian
  expect_equal(tbl[3L, A], "1")  # Black
  expect_equal(tbl[4L, A], "1")  # White
  expect_equal(tbl[4L, B], "2")  # White
})

test_that("calc_counts() handles .total_dt and percentage formatting", {
  tbl <- calc_counts(dt = adsl, target = "RACE", treat = "ARM",
                     .total_dt = adsl_total, indent = "")[[1]]
  expect_match(tbl[2L, A], "0")
  expect_match(tbl[3L, A], "\\(25.0%\\)")
  expect_match(tbl[4L, B], "\\(50.0%\\)")
})

test_that("calc_counts() applies indentation correctly", {
  indent <- "&nbsp;&nbsp;"
  tbl <- calc_counts(dt = adsl, target = "RACE", treat = "ARM", indent = indent)[[1]]
  expect_true(startsWith(tbl$stats[2], indent))
})

test_that("calc_counts() works with logical variables", {
  tbl <- calc_counts(dt = adsl, target = "FLAG", treat = "ARM", indent = "")[[1]]
  expect_true(all(c("FLAG", "TRUE", "FALSE", "NA") %in% tbl$stats))
})

test_that("calc_counts() fails if `dt` is not a data.frame", {
  non_df_inputs <- list(NULL, 42, "not a data frame", matrix(1:4, nrow = 2), list(a = 1, b = 2))

  for (bad_dt in non_df_inputs) {
    expect_error(calc_counts(
      dt = bad_dt,
      target = "RACE",
      treat = "ARM"
    ))
  }
})

test_that("calc_counts() fails if `.total_dt` is not NULL or a data.frame", {
  bad_total_dt_values <- list(
    42,
    "invalid",
    matrix(1:4, nrow = 2),
    list(a = 1, b = 2),
    TRUE
  )

  for (bad_total_dt in bad_total_dt_values) {
    expect_error(calc_counts(
      dt = data.frame(
        USUBJID = 1:2,
        ARM = c("A", "B"),
        RACE = c("X", "Y")
      ),
      target = "RACE",
      treat = "ARM",
      .total_dt = bad_total_dt
    ))
  }
})

test_that("calc_counts() fails if `target` or `treat` are not scalar strings", {
  valid_dt <- data.frame(
    USUBJID = 1:2,
    ARM = c("A", "B"),
    RACE = c("X", "Y"),
    stringsAsFactors = FALSE
  )

  invalid_inputs <- list(
    NULL,
    42,
    TRUE,
    c("A", "B"),
    character(0),
    list("RACE"),
    factor("RACE")
  )

  for (bad_target in invalid_inputs) {
    expect_error(
      calc_counts(dt = valid_dt, target = bad_target, treat = "ARM"),
      regexp = "is_string\\(target\\)"
    )
  }

  for (bad_treat in invalid_inputs) {
    expect_error(
      calc_counts(dt = valid_dt, target = "RACE", treat = bad_treat),
      regexp = "is_string\\(treat\\)"
    )
  }
})

test_that("calc_counts() fails if `indent` is not a scalar string", {
  valid_dt <- data.frame(
    USUBJID = 1:2,
    ARM = c("A", "B"),
    RACE = c("X", "Y"),
    stringsAsFactors = FALSE
  )

  invalid_indent_values <- list(
    NULL,
    1L,
    TRUE,
    character(0),
    c("&nbsp;", "&nbsp;"),
    list("&nbsp;"),
    factor("&nbsp;")
  )

  for (bad_indent in invalid_indent_values) {
    expect_error(
      calc_counts(dt = valid_dt, target = "RACE", treat = "ARM", indent = bad_indent),
      regexp = "is_string\\(indent\\)"
    )
  }
})

test_that("calc_counts() formats percentages correctly with pct_dec = 0", {

  tbl <- calc_counts(
    dt = adsl,
    target = "RACE",
    treat = "ARM",
    .total_dt = adsl_total,
    pct_dec = 0,
    indent = ""
  )[[1]]

  values <- unlist(tbl[-1, -1, drop = FALSE])
  expect_true(all(detect_fmt(values, n_pct_fmt(n = 0L))))
})

test_that("calc_counts() formats percentages correctly with pct_dec = 1", {

  tbl <- calc_counts(
    dt = adsl,
    target = "RACE",
    treat = "ARM",
    .total_dt = adsl_total,
    pct_dec = 1,
    indent = ""
  )[[1]]

  values <- unlist(tbl[-1, -1, drop = FALSE])
  expect_true(all(detect_fmt(values, n_pct_fmt(n = 1L))))
})

test_that("calc_counts() formats percentages correctly with pct_dec = 5", {

  tbl <- calc_counts(
    dt = adsl,
    target = "RACE",
    treat = "ARM",
    .total_dt = adsl_total,
    pct_dec = 5,
    indent = ""
  )[[1]]

  values <- unlist(tbl[-1, -1, drop = FALSE])
  expect_true(all(detect_fmt(values, n_pct_fmt(n = 5L))))
})

vrep <- Vectorize(rep.int)

#' Convert a contingency table to a long-format observation-level data frame
#'
#' [cross_tab_to_obsv_tab()] expands a contingency table or matrix of counts
#' into a long-format data frame where each row represents one observation. The
#' output contains one column per dimension of the input, with repeated rows
#' according to the frequency counts.
#'
#' @param cross_tab A two-way or multi-way contingency table (`matrix` or
#'   `table`) with named `dimnames`. Each combination of factor levels is
#'   assumed to represent a count of occurrences.
#'
#' @param strings_as_factors Should character columns in the output
#'   be converted to factors?
#'
#' @returns A `data.frame` in long format with one row per implied observation
#'   and one column per dimension of the input table.
#'
#' @examples
#' dim_names <- list(Sex = c("Male", "Female"),
#'                   Response = c("Yes", "No"))
#' cross_tab <- matrix(c(2, 1, 3, 4), nrow = 2, dimnames = dim_names)
#' cross_tab_to_obsv_tab(cross_tab)
#'
#' @export
cross_tab_to_obsv_tab <- function(cross_tab, strings_as_factors = TRUE) {
  if (!is.matrix(cross_tab) && !inherits(cross_tab, "table")) {
    stop("Input must be a matrix or a table.")
  }

  comb <- expand.grid(dimnames(cross_tab), stringsAsFactors = strings_as_factors)
  df <- cbind(comb, freq = as.vector(cross_tab))
  idx <- unlist(vrep(seq_len(nrow(df)), df$freq))
  df <- df[idx, names(dimnames(cross_tab))]
  rownames(df) <- NULL
  df
}

#
# `dt`: dummy dataset of three variables (USUBJID, ARM, DTHFL) and 60 subjects
# (observations).
#
DTHFL_Y <- c(18L, 6L, 6L)
DTHFL_N <- c(0L, 6L, 24L)
dim_names <- list(ARM = LETTERS[1:3], DTHFL = c("Y", "N"))
ARM_DTHFL_table <- matrix(c(DTHFL_Y, DTHFL_N), nrow = 3, dimnames = dim_names)

dt <- cross_tab_to_obsv_tab(ARM_DTHFL_table)
dt[["USUBJID"]] <- sprintf("id-%03d", seq_len(nrow(dt)))
dt <- dt[c("USUBJID", "ARM", "DTHFL")]

#
# Tests
#
test_that("`event_count()` can count deaths", {

  sum_stat_tbl_lst <- event_count(
    dt = dt,
    patient = "USUBJID",
    treat = "ARM",
    label = "Deaths",
    .filters = "DTHFL == 'Y'"
  )
  sum_stat_tbl <- sum_stat_tbl_lst[[1L]]

  #
  # Basic assertions on the wrapping list
  #
  expect_equal(class(sum_stat_tbl_lst), "list")
  expect_equal(typeof(sum_stat_tbl_lst), "list")
  expect_equal(length(sum_stat_tbl_lst), 1L)

  #
  # Basic assertions on the data.table within
  #
  expect_s3_class(sum_stat_tbl, "data.table")
  expect_equal(typeof(sum_stat_tbl), "list")
  expect_equal(colnames(sum_stat_tbl), c("stats", dim_names$ARM))

  # Assertions on data.table's contents
  expect_equal(sum_stat_tbl$stats[1L], "Deaths")
  expect_equal(sum_stat_tbl$A[1L], "18 (100.0%)")
  expect_equal(sum_stat_tbl$B[1L], "6 (50.0%)")
  expect_equal(sum_stat_tbl$C[1L], "6 (20.0%)")
})

test_that("`event_count()` can count lives", {

  sum_stat_tbl_lst <- event_count(
    dt = dt,
    patient = "USUBJID",
    treat = "ARM",
    label = "Lives",
    .filters = "DTHFL == 'N'"
  )
  sum_stat_tbl <- sum_stat_tbl_lst[[1L]]

  #
  # Basic assertions on the wrapping list
  #
  expect_equal(class(sum_stat_tbl_lst), "list")
  expect_equal(typeof(sum_stat_tbl_lst), "list")
  expect_equal(length(sum_stat_tbl_lst), 1L)

  #
  # Basic assertions on the data.table within
  #
  expect_s3_class(sum_stat_tbl, "data.table")
  expect_equal(typeof(sum_stat_tbl), "list")
  expect_equal(colnames(sum_stat_tbl), c("stats", dim_names$ARM))

  # Assertions on data.table's contents
  expect_equal(sum_stat_tbl$stats[1L], "Lives")
  expect_equal(sum_stat_tbl$A[1L], "0")
  expect_equal(sum_stat_tbl$B[1L], "6 (50.0%)")
  expect_equal(sum_stat_tbl$C[1L], "24 (80.0%)")
})

# `round_sum_()`: Assumes `x` does not contain NA values.
round_sum_ <- function(x, digits = 0L) {

  stopifnot(
    is.numeric(x),
    is_scalar_number(digits),
    all(x >= 0),
    digits >= 0L
  )

  up <- 10 ^ digits
  x <- x * up
  y <- floor(x)
  indices <- utils::tail(order(x - y), round(sum(x)) - sum(y))
  y[indices] <- y[indices] + 1
  y / up
}

#' Rounds numbers while preserving the total sum
#'
#' [round_sum()] rounds a numeric vector of non-negative values to a specified
#' number of decimal places while ensuring that the sum of the rounded value
#' remains as close as possible to the original total.
#'
#' @param x A numeric vector of non-negative values that you want to round.
#' Missing values (`NA`) are ignored.
#'
#' @param digits The number of decimal places to round to. Default is `0`
#' (integer rounding).
#'
#' @returns A numeric vector of the same length as `x`, with values rounded in
#' such a way that the total sum is preserved.
#'
#' @examples
#' # Rounds to integers, preserving the sum of 100.
#' x <- c(33.3333, 33.3333, 33.3334)
#' (y <- round_sum(x))
#' identical(sum(x), sum(y))
#'
#' # Rounds to integers, preserving the sum of 1002.
#' x <- c(100.5, 200.25, 300.75, 400.5)
#' (y <- round_sum(x))
#' identical(sum(x), sum(y))
#'
#' # Rounds to one decimal place, preserving the total sum.
#' x <- c(12.345, 67.890, 19.765)
#' (y <- round_sum(x))
#' identical(sum(x), sum(y))
#'
#' @export
#'
round_sum <- function(x, digits = 0L) {

  stopifnot(
    is.numeric(x),
    is_scalar_number(digits),
    all(is.na(x) | x >= 0),
    digits >= 0L
  )

  x[!is.na(x)] <- round_sum_(x[!is.na(x)], digits = digits)
  x
}

round_pct_ <- function(
    x,
    digits = 1L,
    method = c("round", "round_sum")) {

  stopifnot(
    is.numeric(x),
    is.numeric(digits),
    all(x >= 0),
    digits >= 0L
  )

  method <- match.arg(method)
  round <- match.fun(method)

  pct <- 100 * x / sum(x)
  rounded_pct <- round(pct, digits = digits)
  rounded_pct
}

#' Rounded percentage
#'
#' [round_pct()] returns the rounded percentages of `x` values.
#'
#' @param x A numeric vector of non-negative values for which you want
#'   percentages to be determined and rounded. Missing values (`NA`) are
#'   ignored.
#'
#' @param digits The number of decimal places to round to. Default is `0`
#' (integer rounding).
#'
#' @param method Rounding method: `"round"` that uses R's base [round()] or
#' `"round_sum"` that uses [dtlg::round_sum][round_sum()].
#'
#' @returns A numeric vector of the same length as `x` with rounded percentages.
#'
#' @examples
#' x <- c(1 / 3, 1 / 3, 1 / 3)
#'
#' # Default method ensures precise rounding but total might not be 100%.
#' round_pct(x = x)
#' sum(round_pct(x = x))
#'
#' # You can trade off rounding precision for precision on the total with the
#' # method `"round_sum"`.
#' round_pct(x = x, method = "round_sum")
#' sum(round_pct(x = x, method = "round_sum"))
#'
#' # Vary the number of decimal places, e.g. increase to three.
#' round_pct(x = x, digits = 3, method = "round_sum")
#'
#' # Missing values are ignored.
#' x <- c(1, 2, NA)
#' round_pct(x = x, digits = 3)
#'
#' @export
#'
round_pct <- function(
    x,
    digits = 1L,
    method = c("round", "round_sum")) {

  stopifnot(
    is.numeric(x),
    is_scalar_number(digits),
    all(is.na(x) | x >= 0),
    digits >= 0L
  )

  method <- match.arg(method)
  x[!is.na(x)] <- round_pct_(x[!is.na(x)], digits = digits, method = method)
  x
}

# round_sum_ ----
test_that("Able to correctly round numbers to specified digits - internal", {
  x <- c(1, 1.2345, 0, 5.001, 10.9994)

  expect_identical(round_sum_(x, 1), c(1.0, 1.2, 0.0, 5.0, 11.0))
  expect_identical(round_sum_(x, 3), c(1.000, 1.235, 0.000, 5.001, 10.999))
})

test_that("Able to correctly round numbers to expected sum - internal", {
  x <- rep(1 / 7, 7L)

  expect_identical(round_sum_(x, 1), rep(c(0.1, 0.2), times = c(4L, 3L)))
  expect_identical(round_sum_(x, 3), c(0.142, rep(0.143, 6L)))
})

test_that("Rounding errors when non-positive numbers are passed - internal", {
  expect_error(round_sum_(-1, 1))
  expect_error(round_sum_(NA_real_, 1))
  expect_error(round_sum_("2.3", 1))
})

# round_sum ----
test_that("Able to correctly round numbers to specified digits", {
  x <- c(1, 1.2345, 0, 5.001, 10.9994)

  expect_identical(round_sum(x, 1), c(1.0, 1.2, 0.0, 5.0, 11.0))
  expect_identical(round_sum(x, 3), c(1.000, 1.235, 0.000, 5.001, 10.999))
})

test_that("Able to correctly round numbers to expected sum", {
  x <- rep(1 / 7, 7L)

  expect_identical(round_sum(x, 1), rep(c(0.1, 0.2), times = c(4L, 3L)))
  expect_identical(round_sum(x, 3), c(0.142, rep(0.143, 6L)))
})

test_that("Able to correctly round numbers with missing values", {
  x <- c(1, 1.2345, 0, NA_real_, 10.9994)

  expect_identical(round_sum(x, 1), c(1.0, 1.2, 0.0, NA_real_, 11.0))
})

test_that("Rounding errors when non-positive numbers are passed", {
  expect_error(round_sum(-1, 1))
  expect_error(round_sum("2.3", 1))
})

# round_pct_ ----
test_that("Able to correctly round percentages to specified digits - internal", {
  x <- c(1.0001, 1.9999, 3, 4)

  expect_identical(round_pct_(x, 1), c(10, 20, 30, 40))
  expect_identical(round_pct_(x, 3), c(10.001, 19.999, 30, 40))
})

test_that("Able to correctly sum rounded percentages to 100", {
  x <- rep(1 / 7, 7L)

  expect_identical(round_pct_(x, 1), rep(14.3, 7L))
  expect_identical(
    round_pct_(x, 1, method = "round_sum"),
    c(14.2, rep(14.3, 6L))
  )
  expect_identical(
    round_pct_(x, 3, method = "round_sum"),
    rep(c(14.285, 14.286), times = c(2, 5))
  )
})

# round_pct ----
test_that("Able to correctly round percentages to specified digits", {
  x <- c(1.0001, 1.9999, 3, 4)

  expect_identical(round_pct(x, 1), c(10, 20, 30, 40))
  expect_identical(round_pct(x, 3), c(10.001, 19.999, 30, 40))
})

#' Count events
#'
#' [event_count()] counts events defined by predicate expressions passed in
#' `.filters`.
#'
#' @inheritParams calc_counts
#'
#' @param patient A string indicating the subject identifying variable.
#'
#' @param label A string to be used as label in the output reporting table. This
#' should be a text descriptive of the event being counted.
#'
#' @param .filters Predicate expressions identifying events in `dt`. Argument
#'   should be passed as a `character` vector of expressions to be evaluated in
#'   the frame of `dt`.
#'
#' @returns A one-element list, where the element is a `data.table`.
#'
#' @examples
#' # Count deaths per arm.
#' event_count(
#'   adsl,
#'   patient = "USUBJID",
#'   treat = "ARM",
#'   label = "Total number of deaths",
#'   .filters = "DTHFL == 'Y'"
#' )[[1]]
#'
#' # Count patients withdraw from study due to an adverse event.
#' withdrawn_lbl <- "Total number of patients withdrawn from study due to an AE"
#' event_count(
#'   adsl,
#'   patient = "USUBJID",
#'   treat = "ARM",
#'   label = withdrawn_lbl,
#'   .filters = "DCSREAS == 'ADVERSE EVENT'"
#' )[[1]]
#'
#' # Count patients with at least one adverse event.
#' # NB: When `.filters` is `NULL` (i.e., omitted), all records in `dt` are used
#' # for counting events.
#' event_count(
#'   adae,
#'   patient = "USUBJID",
#'   treat = "ARM",
#'   label = "Total number of patients with at least one AE",
#'   .filters = "ANL01FL == 'Y'",
#'   .total_dt = adsl
#' )[[1]]
#'
#' @export
#'
event_count <- function(dt,
                        patient,
                        treat,
                        label,
                        .filters = NULL,
                        .total_dt = dt,
                        pct_dec = 1) {

  dt <- maybe_copy_dt(x = dt)
  .total_dt <- maybe_copy_dt(x = .total_dt)

  df_filtered <-
    if (isFALSE(is.null(.filters))) {
      stopifnot(is.character(.filters))
      .filters <- lapply(.filters, str2lang)
      do.call(what = dt_filter, args = c(list(dt = dt), .filters))
    } else {
      dt
    }

  df_filtered_uniq <- unique(df_filtered, by = patient)
  df_filtered_uniq[[label]] <- label
  .total_dt[[label]] <- label

  events <-
    summary_table(
      dt = df_filtered_uniq,
      target = label,
      treat = treat,
      .total_dt = .total_dt,
      indent = ""
    )[.N, ]

  list(events)
}

#' Count total events
#'
#' [total_events()] counts the number of observations in `dt` in each group
#' defined by `treat` levels. Counts are returned in wide format, i.e. one
#' column per level in `treat`.
#'
#' @inheritParams event_count
#'
#' @param dt A `data.frame` containing, at least, the variable indicated in
#'   `treat`.
#'
#' @returns A list wrapping a one-row `data.table` of `1 + n` variables, where
#'   `n` is the number of levels in `treat`. First variable is `stats`,
#'   `character` type, whose value is the argument passed in as `label`.
#'   Following variables are of `integer` type and provide the counts.
#'
#' @examples
#' # In the absence of pre-filtering, `total_events()`, actually, just counts
#' # observations in `dt`.
#' total_events(dt = adsl, treat = "ARM", label = "Subjects")[[1]]
#'
#' # If `dt` is pre-filtered, e.g. with a condition matching an event, then
#' # `total_events()` can be used to (effectively) count events.
#' total_events(dt = adsl[adsl$DTHFL == "Y"], treat = "ARM", label = "Deaths")[[1]]
#'
#' # Another example using the complement predicate condition.
#' total_events(dt = adsl[adsl$DTHFL == "N"], treat = "ARM", label = "Lives")[[1]]
#'
#' @export
#'
total_events <- function(dt, treat, label) {
  dt <- maybe_copy_dt(x = dt)

  j <- dot_wrap(c(n = ".N"))
  by <- as.call(c(quote(list), as.name(treat)))

  dt <- dt[, j = j, by = by, env = list(j = j, by = by)]
  dt <- data.table::transpose(
    dt,
    keep.names = "stats",
    make.names = treat,
    fill = 0
  )
  dt[, `:=`(stats = label)]
  list(dt[])
}


#' Summarise multiple AESI-like events per treatment arm
#'
#' `multi_event_true()` generates a summary table showing the number and
#' percentage of patients with at least one event across multiple binary
#' indicator variables (e.g., flags for adverse events of special interest).
#'
#' Each event is counted only once per patient. This function is typically used
#' for summarising *Adverse Events of Special Interest* (AESIs) or other derived
#' flags (e.g., `SER`, `FATAL`, `RELDSM`) that are binary (TRUE/FALSE).
#'
#' @param dt A `data.frame` or `data.table` containing the binary event flags
#'   and subject-level data.
#'
#' @param event_vars A character vector of column names (binary flags) to
#'   summarise.
#'
#' @param patient A string giving the name of the variable that uniquely
#'   identifies each patient (e.g., `"USUBJID"`).
#'
#' @param treat A string giving the name of the treatment variable (e.g.,
#'   `"ARM"`).
#'
#' @param heading A string to be shown as the first row in the output, usually a
#'   summary descriptor such as `"Total number of patients with at least one"`.
#'
#' @param label Optional. A character vector of the same length as `event_vars`
#'   giving human-readable labels for the output table rows. If `NULL`, labels
#'   are extracted from the `label` attribute of each variable, or fall back to
#'   the variable name.
#'
#' @param .total_dt A `data.frame` or `data.table` containing the total analysis
#'   population (denominator). If `NULL`, `dt` is used as the denominator.
#'
#' @param indent A string to indent the row labels (e.g., `"  "` or `nbsp(n =
#'   4L)` for non-breaking spaces).
#'
#' @param pct_dec An integer indicating how many decimal places to show in
#'   percentages (default is `1`).
#'
#' @returns A one-element list containing a `data.table` with one row per event
#'   plus one header row. The first column is `"stats"` (row labels), and
#'   subsequent columns are one per treatment arm, with values in `"n (x%)"`
#'   format.
#'
#' @examples
#' aesi_vars <- c(
#'   "FATAL", "SER", "SERWD", "SERDSM", "RELSER",
#'   "WD", "DSM", "REL", "RELWD", "RELDSM", "SEV"
#' )
#'
#' heading <- "Total number of patients with at least one AE"
#'
#' multi_event_true(
#'   dt = aesi,
#'   event_vars = aesi_vars,
#'   patient = "USUBJID",
#'   treat = "ARM",
#'   heading = heading,
#'   .total_dt = adsl,
#'   indent = "  "
#' )[[1]]
#'
#' @seealso [event_count()], [total_events()]
#'
#' @export
#'
multi_event_true <- function(dt,
                             event_vars,
                             patient,
                             treat,
                             heading,
                             label = NULL,
                             .total_dt = NULL,
                             indent = nbsp(n = 4L),
                             pct_dec = 1) {
  dt <- maybe_copy_dt(x = dt)
  event_filters <- paste0(event_vars, " == TRUE")
  event_label <- label %||% lapply(event_vars, \(x) label(dt[[x]]) %||% x)

  stopifnot(
    identical(length(event_filters), length(event_label)),
    all(event_vars %in% colnames(dt))
  )

  event <- mapply(
    event_count,
    .filters = event_filters,
    label = event_label,
    MoreArgs = list(
      dt = dt,
      patient = patient,
      treat = treat,
      .total_dt = .total_dt,
      pct_dec = pct_dec
    )
  )

  event <- data.table::rbindlist(event, use.names = TRUE)
  target_rows <- event$stats
  event <- rbind(rep(list(""), times = ncol(event)), event)
  event[, `:=`(stats = c(heading, paste0(indent, target_rows)))]

  list(event[])
}


#' Summarise adverse events by arm and other grouping variables
#'
#' `event_count_by()` creates a tabular summary of adverse events grouped by a
#' higher-level classification variable (e.g., system organ class), and counts
#' both the number of events and the number of unique patients per treatment arm.
#'
#' @param dt A `data.frame` or `data.table` containing the adverse event data
#'   and patient-level identifiers.
#'
#' @param patient A string giving the name of the patient identifier variable
#'   (e.g., `"USUBJID"`).
#'
#' @param treat A string giving the name of the treatment arm variable (e.g.,
#'   `"ARM"`).
#'
#' @param rows_by A string giving the name of the grouping variable (e.g.,
#'   `"AEBODSYS"` for body system).
#'
#' @param target A string giving the name of the variable to report within each
#'   group (e.g., `"AEDECOD"` for preferred term).
#'
#' @param .total_dt A `data.frame` or `data.table` containing the denominator
#'   population. Defaults to `dt`.
#'
#' @param indent A string used to indent row labels (e.g., `"  "` or `nbsp(n =
#'   4L)`).
#'
#' @param pct_dec Integer. Number of decimal places to show in percentages.
#'   Defaults to `1`.
#'
#' @returns A `data.table` with the following structure:
#'
#' * One row per combination of `rows_by` and `target`
#' * One row per group total (`Total number of events`)
#' * One row per patient-level total (`Total number of patients with at least one event`)
#'
#' Columns include:
#' - `stats`: character column with labels
#' - one column per level of the `treat` variable, formatted as `"n (x%)"`
#'
#' @examples
#' event_count_by(
#'   dt = adae[adae$ANL01FL == "Y"],
#'   patient = 'USUBJID',
#'   treat = 'ARM',
#'   rows_by = 'AEBODSYS',
#'   target = 'AEDECOD',
#'   .total_dt = adsl,
#'   indent = ' '
#' )
#'
#' @seealso [event_count()], [calc_stats()], [total_events()]
#'
#' @export
#'
event_count_by <- function(dt,
                           patient,
                           treat,
                           rows_by,
                           target,
                           .total_dt = dt,
                           indent = nbsp(n = 4L),
                           pct_dec = 1) {

  dt <- maybe_copy_dt(x = dt)
  .total_dt <- maybe_copy_dt(x = .total_dt)

  event <- dt

  event_split <- split(droplevels(event), by = rows_by, drop = TRUE)

  event_patient <- mapply(
    event_count,
    dt = event_split,
    treat = treat,
    label = "Total number of patients with at least one event",
    MoreArgs = list(
      .total_dt = .total_dt,
      patient = patient,
      pct_dec = pct_dec
    )
  )
  event_total <- mapply(total_events, event_split, treat = treat, label = "Total number of events")
  event_target <- mapply(
    calc_stats,
    dt = event_split,
    target = target,
    treat = treat,
    indent = indent,
    target_name = names(event_split),
    MoreArgs = list(.total_dt = .total_dt, pct_dec = pct_dec)
  )
  event_table <- mapply(list, event_target, event_total, event_patient)
  event_table <- data.table::rbindlist(event_table, use.names = TRUE)
  event_table
}

#' Extract per-column level sets from a data.frame
#'
#' For each column in `df`, returns the set of values used to define
#' group spaces. Factors return their full `levels()` (including unused) or
#' only *used* levels, as controlled by `.fct_levels`. Non-factors return
#' unique values, optionally sorted.
#'
#' @param df A data.frame.
#' @param cols Optional character vector of column names to restrict
#'   the output. If `NULL` (default), all columns are used.
#' @param .sort Logical (default `TRUE`). If `TRUE`, non-factor
#'   columns use `sort(unique(x), na.last = TRUE)`; if `FALSE`, they use
#'   `unique(x)` (order of first appearance). Ignored for factor columns.
#' @param .fct_levels One of `"used"` or `"all"` (default `"all"`). Controls
#'   whether factor columns return only levels that appear in the data or all
#'   declared levels. The order of returned factor levels always follows the
#'   factor's level order.
#'
#' @returns A named list with one element per selected column of `df`.
#'   For factors, the element is a character vector of levels; otherwise it is
#'   the unique values (with `NA` kept last when `.sort = TRUE`).
#'
#' @examples
#' df <- data.frame(
#'   i = c(2L, 1L, 2L),
#'   c = c("b", "a", "b"),
#'   f = factor(c("low", "med", "low"),
#'              levels = c("low", "med", "high", "very high")),
#'   l = c(TRUE, NA, FALSE)
#' )
#'
#' # All columns
#' level_set(df)
#'
#' # Subset of columns
#' level_set(df, cols = c("i", "f"))
#'
#' # Keep order of first appearance for non-factors
#' level_set(df, .sort = FALSE)
#'
#' # Factors: only used levels vs all declared levels
#' level_set(df, cols = "f", .fct_levels = "used")
#' level_set(df, cols = "f", .fct_levels = "all")
#'
#' @noRd
level_set <- function(df, cols = NULL, .sort = TRUE, .fct_levels = c("all", "used")) {
  stopifnot(is.data.frame(df))
  .fct_levels <- match.arg(.fct_levels)

  df_cols <- colnames(df)
  cols <- cols %||% df_cols
  stopifnot(all(cols %in% df_cols))

  out <- lapply(cols, function(col) {
    x <- df[[col]]
    if (is.factor(x)) {
      levs <- levels(x)
      if (.fct_levels == "all") {
        levs
      } else {
        # keep only levels that actually appear, in the factor's level order
        levs[levs %in% as.character(x)]
      }
    } else {
      if (.sort) sort(unique(x), na.last = TRUE) else unique(x)
    }
  })
  stats::setNames(out, cols)
}

test_that("factors return full levels including unused", {
  df <- data.frame(
    f = factor(c("low", "med", "low"),
               levels = c("low", "med", "high", "very high"))
  )
  out <- level_set(df)
  expect_named(out, "f")
  expect_identical(out$f, c("low", "med", "high", "very high"))
})

test_that("non-factors sorted keep NA at end", {
  df <- data.frame(
    i = c(3L, NA, 1L, 2L, 1L),
    c = c("b", "a", NA, "b", "c"),
    l = c(TRUE, NA, FALSE, TRUE, FALSE)
  )
  out <- level_set(df, .sort = TRUE)

  expect_identical(out$i, c(1L, 2L, 3L, NA))
  expect_identical(out$c, c("a", "b", "c", NA))
  # For logical, FALSE < TRUE; NA last.
  expect_identical(out$l, c(FALSE, TRUE, NA))
})

test_that("non-factors unsorted preserve first-appearance order incl. NA", {
  df <- data.frame(
    i = c(3L, NA, 1L, 2L, 1L),
    c = c("b", "a", NA, "b", "c"),
    l = c(TRUE, NA, FALSE, TRUE, FALSE)
  )
  out <- level_set(df, .sort = FALSE)

  expect_identical(out$i, c(3L, NA, 1L, 2L))
  expect_identical(out$c, c("b", "a", NA, "c"))
  expect_identical(out$l, c(TRUE, NA, FALSE))
})

test_that("cols subset works and errors on invalid names", {
  df <- data.frame(
    a = 1:3,
    b = factor(c("x", "y", "x"), levels = c("x", "y", "z"))
  )
  out <- level_set(df, cols = c("b"))
  expect_named(out, "b")
  expect_identical(out$b, c("x", "y", "z"))

  expect_error(level_set(df, cols = c("a", "nope")))
})

test_that("empty cols produces empty named list", {
  df <- data.frame(a = 1:3)
  out <- level_set(df, cols = character(0))
  expect_true(is.list(out) && length(out) == 0L)
  expect_identical(names(out), character(0))
})

test_that("factor levels: used vs all", {
  df <- data.frame(f = factor(
    c("low", "med", "low"),
    levels = c("low", "med", "high", "very high")
  ))
  used <- level_set(df, cols = "f", .fct_levels = "used")$f
  allv <- level_set(df, cols = "f", .fct_levels = "all")$f

  expect_identical(used, c("low", "med"))
  expect_identical(allv, c("low", "med", "high", "very high"))
})

test_that(".sort ignored for factors; respected for non-factors", {
  df <- data.frame(
    f = factor(c("b", "a", "b", "b", "a"), levels = c("a", "b", "c")),
    i = c(3L, NA, 1L, 2L, 1L)
  )

  # factors follow level order regardless of .sort
  expect_identical(level_set(df, cols = "f", .sort = TRUE)$f, c("a", "b", "c"))
  expect_identical(level_set(df, cols = "f", .sort = FALSE)$f, c("a", "b", "c"))

  # non-factors: .sort controls ordering and NA placement
  expect_identical(level_set(df, cols = "i", .sort = TRUE)$i, c(1L, 2L, 3L, NA))
  expect_identical(level_set(df, cols = "i", .sort = FALSE)$i, c(3L, NA, 1L, 2L))
})

#' Expand grid as data.table from named vectors
#'
#' @description
#' `dt_expand_grid()` creates the Cartesian product of the named vectors
#' supplied via `...`, preserving their type, values, and order.
#' If `.include_na = TRUE`, `NA` is appended to each vector if it is not
#' already present.
#'
#' @param ... Named vectors. Each name becomes a column name in the result.
#' @param .include_na Logical, default `FALSE`. If `TRUE`, append `NA` to each
#'   input vector if it is not already present.
#'
#' @returns A `data.table` with one column per input and all combinations of
#' values.
#'
#' @noRd
#' @keywords internal
dt_expand_grid <- function(..., .include_na = FALSE) {
  args <- list(...)
  if (length(args) == 0L) return(data.table::data.table())
  if (is.null(nms <- names(args)) || any(!nzchar(nms))) {
    stop("All arguments to dt_expand_grid() must be *named*.")
  }

  if (.include_na) {
    args <- lapply(args, function(v) {
      if (!anyNA(v)) c(v, NA) else v
    })
  }

  grid <- do.call(
    data.table::CJ,
    c(args, list(sorted = FALSE, unique = FALSE))
  )
  data.table::setnames(grid, nms)
  grid[]
}

#' Count the observations in each group
#'
#' `dt_count()` is a simple wrapper around `data.table` commands to perform
#' a count on observations in each group. Groups are defined by indicating
#' variables to group by.
#'
#' @param dt A `data.table` or an object coercible to a `data.table`.
#' @param ... Variables to group by, passed as `character`. If named arguments
#' are passed, then those names become the new variable names.
#' @param .name The name of the new column in the output.
#' If omitted, it will default to `n`.
#' @param .fct_levels One of `"used"` or `"all"` (default `"all"`). Controls
#'   whether factor columns return only levels that appear in the data or all
#'   declared levels. The order of returned factor levels always follows the
#'   factor's level order.
#' @param .include_na Whether to include `NA` as unobserved levels.
#'
#' @returns A `data.table` with summarised counts per group.
#'
#' @examples
#' x <- c("A", "A", "A", "B", "B", "B")
#' y <- c("X", "Y", "Z", "Z", "Z", NA)
#' (dt <- data.table::data.table(x, y))
#'
#' # Count observations grouped by `x`
#' dt_count(dt = dt, "x")
#'
#' # Count observations grouped by `y`
#' dt_count(dt = dt, "y")
#'
#' # Count observations grouped by `x` and `y`
#' dt_count(dt = dt, "x", "y")
#'
#' # Count observations grouped by `x` and `y` and rename them to `x1` and `y2`,
#' # respectively.
#' dt_count(dt = dt, x1 = "x", y1 = "y")
#'
#' # Name the new column `total` instead of `n`.
#' dt_count(dt = dt, x1 = "x", y1 = "y", .name = "total")
#'
#' @noRd
#' @keywords internal
dt_count <- function(dt,
                     ...,
                     .name = "n",
                     .fct_levels = c("all", "used"),
                     .include_na = FALSE) {

  .fct_levels <- match.arg(.fct_levels)
  group_vars <- list(...)
  dt <- maybe_copy_dt(x = dt)

  # Original vs output names (support renaming)
  orig <- unlist(group_vars)
  out_names <- names(group_vars)
  if (is.null(out_names)) out_names <- rep("", length(orig))
  out_names[out_names == ""] <- orig[out_names == ""]

  if (.name %in% out_names) {
    stop("`.name` ('", .name, "') conflicts with a grouping column name.", call. = FALSE)
  }

  # Fast count of observed groups
  by <- dot_wrap(unname(orig))
  out <- dt[, .N, by = by, env = list(by = by)]
  data.table::setnames(out, old = orig, new = out_names)
  data.table::setnames(out, "N", .name)

  vals_list <- level_set(df = dt, cols = orig, .sort = TRUE, .fct_levels = .fct_levels)
  names(vals_list) <- out_names

  # Full key space via dt_expand_grid(); optionally append NA per column
  key_space <- do.call(
    dt_expand_grid,
    c(vals_list, list(.include_na = .include_na))
  )

  # Left join counts into full key space; fill missing with 0
  res <- out[key_space, on = out_names]
  res[is.na(get(.name)), (.name) := 0L][]
}

#' Summarise each group down to one row
#'
#' `dt_summarise()` summarises a table using `data.table` as backend. Grouping
#' is indicated with `.by` and aggregations as expressions in `...`, evaluated
#' in the frame of `dt`.
#'
#' @param dt A `data.frame`.
#'
#' @param .by A character vector of variable names to be used as grouping
#' criteria.
#'
#' @param ... A sequence of expressions evaluated in the frame of `dt`.
#'
#' @param .env A list or environment for substitution of variables in `j`
#' and `by` parameters.
#'
#' @returns A `data.table` with summary statistics.
#'
#' @noRd
#' @keywords internal
#'
dt_summarise <- function(dt, .by, ..., .env = parent.frame()) {

  summarising_mappings <- as.list(substitute(...()))
  dt <- maybe_copy_dt(x = dt)
  data.table::setkeyv(x = dt, cols = .by)

  j <- substitute_q(as.call(c(quote(.), summarising_mappings)), env = .env)
  by <- as.call(c(quote(.), lapply(.by, as.name)))
  drop_keys(dt[, j = j, by = by, env = list(j = j, by = by)])
}

dt_filter <- function(dt, ..., .env = parent.frame()) {
  filters <- as.list(substitute(...()))
  i <- substitute_q(as.call(c(quote(and), filters)), env = .env)
  dt[i, , env = list(i = i)]
}

test_that("`dt_expand_grid()` basic two-vector grid preserves order and duplicates", {
  a <- c("x", "x", "y")
  b <- c(1L, 2L)

  g <- dt_expand_grid(a = a, b = b)

  # dimensions are length(a) * length(b)
  expect_equal(nrow(g), length(a) * length(b))
  expect_identical(names(g), c("a", "b"))

  # Expected row-wise expansion (no dedup, order preserved)
  expect_identical(as.character(g$a), rep(a, each = length(b)))
  expect_identical(g$b, rep(b, times = length(a)))
})

test_that("`dt_expand_grid()` preserves types: integer, double, logical, character", {
  a <- 1:3              # integer
  b <- c(0.5, 1.5)      # double
  c <- c(TRUE, FALSE)   # logical
  d <- c("a", "b")      # character

  g <- dt_expand_grid(a = a, b = b, c = c, d = d)

  expect_identical(vapply(g, class, character(1)),
                   c(a = "integer", b = "numeric", c = "logical", d = "character"))
  expect_equal(nrow(g), length(a) * length(b) * length(c) * length(d))
})

test_that("`dt_expand_grid()` preserves factor and ordered factor with level order", {
  f  <- factor(c("low", "med", "high"), levels = c("low", "med", "high"), ordered = FALSE)
  fo <- factor(c("A", "B"), levels = c("A", "B"), ordered = TRUE)

  g <- dt_expand_grid(severity = f, group = fo)

  # Classes preserved
  expect_true(is.factor(g$severity))
  expect_true(is.ordered(g$group))

  # Level orders preserved exactly
  expect_identical(levels(g$severity), c("low", "med", "high"))
  expect_identical(levels(g$group), c("A", "B"))

  # Row count as product of input lengths (not unique levels)
  expect_equal(nrow(g), length(f) * length(fo))
})

test_that("`dt_expand_grid()`: .include_na appends NA only when absent", {
  a <- c("x", "y")
  b <- c(1L, 2L)

  g1 <- dt_expand_grid(a = a, b = b, .include_na = TRUE)
  expect_true(anyNA(g1$a))
  expect_true(anyNA(g1$b))

  # If NA already present, do not add extra
  a2 <- c("x", NA)
  g2 <- dt_expand_grid(a = a2, b = b, .include_na = TRUE)
  expect_equal(sum(is.na(g2$a)), length(unique(g2$b))) # exactly one NA per b value
})

test_that("`dt_expand_grid()` works with a single input vector", {
  a <- c("x", "y", "z")
  g <- dt_expand_grid(a = a)
  expect_identical(g$a, a)
  expect_equal(nrow(g), length(a))
})

test_that("`dt_expand_grid()` returns empty data.table when called with no arguments", {
  g <- dt_expand_grid()
  expect_s3_class(g, "data.table")
  expect_equal(nrow(g), 0L)
  expect_equal(ncol(g), 0L)
})

test_that("`dt_expand_grid()` throws error when arguments are unnamed or have empty names", {
  expect_error(dt_expand_grid(c("a", "b")), "must be \\*named\\*")
  expect_error({
    v <- c("a", "b")
    dt_expand_grid(v)
  }, "must be \\*named\\*")
})

df <- data.frame(
  int_var  = rep(1:6, each = 3)[1:18],
  char_var = rep(c("A", "B", "C"), times = 6),
  fac_var  = factor(
    rep(c("low", "med", "high"), times = 6)[1:18],
    levels = c("low", "med", "high", "very high")  # "very high" unused
  ),
  log_var  = c(TRUE, FALSE, NA, TRUE, FALSE, TRUE,
               FALSE, TRUE, NA, TRUE, TRUE, FALSE,
               FALSE, NA, TRUE, FALSE, TRUE, TRUE)
)

test_that("single group: int_var (default name 'n')", {
  res <- dt_count(df, "int_var")
  expect_s3_class(res, "data.table")
  expect_identical(names(res), c("int_var", "n"))

  expect_equal(res$int_var, 1:6)
  expect_equal(res$n, rep(3L, 6))          # each appears 3 times
  expect_true(is.integer(res$n))
})

test_that("single group: char_var (default name 'n')", {
  res <- dt_count(df, "char_var")

  expect_identical(names(res), c("char_var", "n"))
  expect_equal(res$char_var, c("A", "B", "C"))
  expect_equal(res$n, rep(6L, 3))          # each appears 6 times
})

test_that("single group: fac_var without `.fct_levels = 'used'` excludes unused level", {
  res <- dt_count(df, "fac_var", .fct_levels = "used")

  expect_identical(names(res), c("fac_var", "n"))
  expect_equal(as.character(res$fac_var), c("low", "med", "high")) # no "very high"
  expect_false(any(as.character(res$fac_var) == "very high"))
  expect_equal(sort(res$n), rep(6L, 3))
})

test_that("single group: fac_var with `.fct_levels = 'all'` includes unused level with 0", {
  res <- dt_count(df, "fac_var", .fct_levels = "all")

  expect_identical(names(res), c("fac_var", "n"))
  expect_equal(as.character(res$fac_var), c("low", "med", "high", "very high"))
  expect_true(res[fac_var == "very high", n][[1]] == 0L)
  expect_true(is.integer(res$n))
})

test_that("Enabling NAs to be included in factor counts adds extra 'missing' row to table", {
  # fac_var has no NA observed; .include_na = TRUE should add NA with 0
  res <- dt_count(df, "fac_var", .fct_levels = "all", .include_na = TRUE)

  expect_true(any(is.na(res$fac_var)))
  expect_equal(res[is.na(fac_var), n][[1]], 0L)
})

test_that("single group: log_var includes TRUE, FALSE, and NA counts", {
  res <- dt_count(df, "log_var")

  # Expected counts from df definition:
  # TRUE = 9, FALSE = 6, NA = 3
  expect_equal(res[log_var == TRUE, n][[1]], 9L)
  expect_equal(res[log_var == FALSE, n][[1]], 6L)
  expect_equal(res[is.na(log_var), n][[1]], 3L)
})

test_that("single group: .fct_levels = 'all' with non-factor (int_var) keeps uniques", {
  res1 <- dt_count(df, "int_var", .fct_levels = "all")
  expect_equal(res1$int_var, 1:6)
  expect_false(any(is.na(res1$int_var)))
})

test_that("Enabling NAs to be included in non-factor counts adds extra 'missing' row to table", {
  res2 <- dt_count(df, "int_var", .fct_levels = "all", .include_na = TRUE)
  expect_true(any(is.na(res2$int_var)))
  expect_equal(res2[is.na(int_var), n][[1]], 0L)
})

test_that("single group: custom .name works", {
  res <- dt_count(df, "char_var", .name = "Nobs")
  expect_identical(names(res), c("char_var", "Nobs"))
  expect_true(is.integer(res$Nobs))
})

test_that("single group: renaming grouping variable works", {
  res <- dt_count(df, grp = "int_var")
  expect_identical(names(res), c("grp", "n"))

  expect_equal(res$grp, 1:6)
  expect_equal(res$n, rep(3L, 6))
})

test_that("multi-group: int_var x char_var counts are correct", {
  res <- dt_count(df, "int_var", "char_var")
  # For this construction, each (int, char) appears exactly once
  expect_equal(nrow(res), 18L)
  expect_true(all(res$n == 1L))
})

test_that("multi-group: char_var x fac_var with .fct_levels='all' completes cartesian space", {
  res <- dt_count(df, "char_var", "fac_var", .fct_levels = "all")
  # 3 chars * 4 factor levels = 12 rows
  expect_equal(nrow(res), 12L)

  # Only these pairs are observed in df: (A,low), (B,med), (C,high) — each with 6
  expect_equal(res[char_var == "A" & fac_var == "low", n][[1]], 6L)
  expect_equal(res[char_var == "B" & fac_var == "med", n][[1]], 6L)
  expect_equal(res[char_var == "C" & fac_var == "high", n][[1]], 6L)

  # Unused level 'very high' should be present with 0
  expect_equal(res[fac_var == "very high", n] |> as.integer(), rep(0L, 3))
  expect_true(is.integer(res$n))
})

test_that("multi-group: add NA via .include_na across cartesian space", {
  res <- dt_count(df, "char_var", "fac_var",
                  .fct_levels = "all", .include_na = TRUE)
  # char gains an NA level; factor has 4 levels, plus NA → 4 * 5 = 20
  expect_equal(nrow(res), 20L)
  expect_true(any(is.na(res$char_var)))
  # The NA x fac_var rows should all be zero
  expect_true(all(res[is.na(char_var), n] == 0L))
})

test_that("renaming two grouping variables works", {
  res <- dt_count(df, i = "int_var", f = "fac_var", .fct_levels = "used")
  expect_identical(names(res), c("i", "f", "n"))
  # Only three observed pairs by construction
  expect_equal(nrow(res), 18L)  # 6 ints * 3 used fac levels = 18 rows
})

test_that("error on non-existent columns", {
  expect_error(dt_count(df, "no_such_col"), regexp = "no_such_col")
})

test_that("defensive: duplicate grouping columns should error", {
  # If you decide to support this, change this test; for now, expect error.
  expect_error(dt_count(df, "int_var", a = "int_var"))
})

test_that(".name collision with existing column is handled", {
  # Decide your policy. If you want to forbid, change dt_count to check and stop().
  # Here we expect an error to be safer.
  expect_error(dt_count(df, "char_var", .name = "char_var"))
})

test_that(".include_na does not duplicate NA rows when NA already present", {
  # log_var already has NA; .include_na should not create a second NA row
  res1 <- dt_count(df, "log_var")
  res2 <- dt_count(df, "log_var", .include_na = TRUE)
  n_na1 <- res1[is.na(log_var), n][[1]]
  n_na2 <- res2[is.na(log_var), n][[1]]
  expect_equal(n_na1, 3L)
  expect_equal(n_na2, 3L)
})

test_that("zeros introduced by completion remain integer", {
  res <- dt_count(df, "fac_var", .fct_levels = "all", .include_na = TRUE)
  expect_true(is.integer(res$n))
  expect_true(all(res[fac_var == "very high" | is.na(fac_var), n] == 0L))
})

test_that("non-factors: Inf and NaN are retained; NaN treated as NA in sort()", {
  df2 <- data.frame(x = c(1, Inf, NaN, 1))
  res <- dt_count(df2, "x", .fct_levels = "all", .include_na = TRUE)
  # Expect rows for 1, Inf, and NA (NaN merges into NA via sort/unique semantics)
  expect_true(all(c(1, Inf) %in% res$x))
  expect_true(any(is.na(res$x)))
})

dot_wrap <- function(x) {
  stopifnot(is.character(x))
  as.call(c(str2lang("."), lapply(x, as.name)))
}

drop_keys <- function(dt) {
  data.table::setkey(x = dt, NULL)[]
}


#' Relevel a single column in a data.table (preserving ordered state)
#'
#' @description
#' `dt_relevel_col_()` enforces a specific level order for one column of a
#' `data.table`. If the column is already an ordered factor, its ordered
#' state is preserved; otherwise, the result is an unordered factor. Values not
#' present in `levels` are silently converted to `NA` (base R behaviour).
#'
#' @param dt A `data.table` or an object coercible to one.
#'
#' @param col A single column name (string) to relevel.
#'
#' @param levels A character vector with the desired target level order. Any
#'   duplicates are ignored (first occurrence kept). Levels not listed are
#'   dropped; observed values outside this set become `NA`.
#'
#' @returns A `data.table`.
#'
#' @examples
#'
#' dt <- data.table(
#'   grp = factor(c("b", "a", "b", "c"), levels = c("a", "b", "c", "d")),
#'   band = c("low", "high", "med", "high")
#' )
#'
#' # Relevel 'grp' to c("c", "a"); values not in the set become NA
#' res <- dt_relevel_col_(dt, col = "grp", levels = c("c", "a"))
#' levels(res$grp)
#'
#' # In-place modification
#' invisible(dt_relevel_col_(dt, col = "band", levels = c("high", "low")))
#' levels(dt$band)
#'
#' @noRd
#' @keywords internal
#'
dt_relevel_col_ <- function(dt, col, levels) {
  stopifnot(length(col) == 1L, is.character(col), !is.na(col))
  stopifnot(is.character(levels))

  out <- dt

  if (!col %in% names(out))
    stop(sprintf("Column '%s' not found in data.", col))

  target_levels <- unique(levels)
  cur <- out[[col]]
  ord_flag <- is.ordered(cur)  # FALSE for non-factors; TRUE only for ordered factors

  # Skip if levels and ordered flag already match
  matching_levels <- is.factor(cur) &&
    identical(levels(cur), target_levels) &&
    identical(is.ordered(cur), ord_flag)
  if (matching_levels) {
    return(out)
  }

  out[, (col) := factor(.SD[[1L]], levels = target_levels, ordered = ord_flag), .SDcols = col]

  out
}

#' Relevel multiple columns in a data.table
#'
#' @param dt A `data.table` (or coercible).
#'
#' @param levels Named list: each name is a column; each value is a
#'   character vector with the desired target level order for that column.
#'
#' @returns A `data.table` (copy or modified in place).
#' @noRd
#' @keywords internal
dt_relevel_col <- function(dt, levels) {
  stopifnot(is.list(levels), length(levels) > 0L, !is.null(names(levels)))
  out <- maybe_copy_dt(x = dt)

  cols <- names(levels)

  for (c in cols) {
    dt_relevel_col_(out, col = c, levels = levels[[c]])
  }

  out
}

test_that("relevels a factor column and enforces level order", {
  dt <- data.table::data.table(g = factor(c("b", "a", "b", "c"), levels = c("a", "b", "c", "d")))
  res <- dt_relevel_col_(dt, col = "g", levels = c("c", "a"))

  expect_s3_class(res, "data.table")
  expect_identical(levels(res$g), c("c", "a"))
  # Values not in levels -> NA
  expect_equal(as.character(res$g), c(NA, "a", NA, "c"))
  expect_false(is.ordered(res$g)) # original was not ordered
})

test_that("coerces non-factor columns and converts out-of-set to NA", {
  dt <- data.table::data.table(h = c("low", "med", "high", "med"))
  res <- dt_relevel_col_(dt, col = "h", levels = c("high", "low"))

  expect_identical(levels(res$h), c("high", "low"))
  expect_equal(as.character(res$h), c("low", NA, "high", NA))
  expect_false(is.ordered(res$h))
})

test_that("preserves ordered flag when input is an ordered factor", {

  x <- factor(c("b", "a", "b"), levels = c("a", "b", "c"), ordered = TRUE)
  dt <- data.table::data.table(g = x)
  res <- dt_relevel_col_(dt, col = "g", levels = c("b", "a"))

  expect_true(is.ordered(res$g))
  expect_identical(levels(res$g), c("b", "a"))
})

test_that("duplicate levels in argument are de-duplicated", {
  dt <- data.table::data.table(g = factor(c("a", "b", "a")))
  res <- dt_relevel_col_(dt, col = "g", levels = c("b", "b", "a", "a"))

  expect_identical(levels(res$g), c("b", "a"))
})

test_that("dt_relevel_col_() always modifies by reference", {

  dt <- data.table::data.table(g = factor(c("a", "b", "a"), levels = c("a", "b", "c")))
  res <- dt_relevel_col_(dt, col = "g", levels = c("b", "a"))

  # original changed
  expect_identical(levels(dt$g), c("b", "a"))
  # result changed
  expect_identical(levels(res$g), c("b", "a"))
})

test_that("idempotent when levels already match and order preserved", {

  dt <- data.table::data.table(g = factor(c("b", "a", "b"), levels = c("b", "a")))
  res <- dt_relevel_col_(dt, col = "g", levels = c("b", "a"))

  expect_identical(levels(res$g), c("b", "a"))
  expect_equal(as.character(res$g), c("b", "a", "b"))
})

test_that("errors when column is missing", {
  dt <- data.table::data.table(g = factor(c("a", "b")))
  expect_error(
    dt_relevel_col_(dt, col = "h", levels = c("a", "b")),
    "Column 'h' not found"
  )
})

test_that("augmented level set: extra levels are added but data unchanged", {

  set_dt_copy_semantics("value")
  # original factor (unordered), observed levels: a, b
  dt <- data.table::data.table(g = factor(c("a", "b", "a"), levels = c("a", "b")))
  before_na <- sum(is.na(dt$g))

  # provide augmented target levels (includes 'c' and 'd', not present in data)
  target <- c("b", "a", "c", "d")
  res <- dt_relevel_col_(dt, col = "g", levels = target)

  # levels now match augmented set, in the requested order
  expect_identical(levels(res$g), target)

  # underlying data values remain the same (no new values introduced)
  expect_identical(as.character(res$g), c("a", "b", "a"))

  # NA count is unchanged
  expect_identical(sum(is.na(res$g)), before_na)

  # unordered preserved (input was not ordered)
  expect_false(is.ordered(res$g))
})

test_that("augmented level set preserves ordered flag for ordered factors", {


  # ordered factor, observed levels: low, high
  set_dt_copy_semantics("reference")
  x <- factor(c("low", "high", "low"), levels = c("low", "high"), ordered = TRUE)
  dt <- data.table::data.table(h = x)

  # augmented levels include 'medium' and 'very_high' (not observed)
  set_dt_copy_semantics("value")
  target <- c("high", "low", "medium", "very_high")
  res <- dt_relevel_col_(dt, col = "h", levels = target)

  # levels updated to augmented set in requested order
  expect_identical(levels(res$h), target)

  # data values unchanged
  expect_identical(as.character(res$h), c("low", "high", "low"))

  # ordered flag preserved
  expect_true(is.ordered(res$h))
})

test_that("relevels multiple columns with specified target levels", {

  dt <- data.table::data.table(
    g = factor(c("b", "a", "b", "c"), levels = c("a", "b", "c", "d")),
    h = c("low", "high", "med", "high")
  )

  res <- dt_relevel_col(
    dt,
    levels = list(g = c("c", "a"), h = c("high", "low"))
  )

  expect_s3_class(res, "data.table")
  expect_identical(levels(res$g), c("c", "a"))
  expect_identical(levels(res$h), c("high", "low"))

  # Data re-mapped / NA as expected
  expect_equal(as.character(res$g), c(NA, "a", NA, "c"))
  expect_equal(as.character(res$h), c("low", "high", NA, "high"))
})

test_that("augmented level sets are allowed and data remain unchanged", {

  dt <- data.table::data.table(
    g = factor(c("a", "b", "a"), levels = c("a", "b")),
    h = factor(c("low", "high", "low"), levels = c("low", "high"))
  )

  target <- list(g = c("b", "a", "c", "d"), h = c("high", "low", "medium"))
  res <- dt_relevel_col(dt, levels = target)

  expect_identical(levels(res$g), target$g)
  expect_identical(levels(res$h), target$h)
  expect_identical(as.character(res$g), c("a", "b", "a"))
  expect_identical(as.character(res$h), c("low", "high", "low"))
})

test_that("preserves ordered flag per column where applicable", {

  dt <- data.table::data.table(
    g = factor(c("b", "a", "b"), levels = c("a", "b"), ordered = TRUE),
    h = c("low", "high", "low")  # not ordered
  )

  res <- dt_relevel_col(dt, levels = list(g = c("b", "a"), h = c("high", "low")))
  expect_true(is.ordered(res$g))
  expect_false(is.ordered(res$h))
})

test_that("coerces non-factor columns and converts out-of-set to NA", {

  dt <- data.table::data.table(
    g = c("b", "a", "b", "c"),
    h = c("low", "high", "med", "high")
  )

  res <- dt_relevel_col(dt, levels = list(g = c("c", "a"), h = c("high", "low")))
  expect_identical(levels(res$g), c("c", "a"))
  expect_identical(levels(res$h), c("high", "low"))

  expect_equal(as.character(res$g), c(NA, "a", NA, "c"))
  expect_equal(as.character(res$h), c("low", "high", NA, "high"))
})

test_that("value semantics returns a modified copy and leaves input unchanged", {

  dt <- data.table::data.table(
    g = factor(c("a", "b", "a"), levels = c("a", "b", "c")),
    h = c("low", "high", "low")
  )

  set_dt_copy_semantics("value")
  res <- dt_relevel_col(dt, levels = list(g = c("b", "a"), h = c("high", "low")))

  # original unchanged
  expect_identical(levels(dt$g), c("a", "b", "c"))
  expect_null(attr(dt$h, "levels")) # still character

  # result changed
  expect_identical(levels(res$g), c("b", "a"))
  expect_identical(levels(res$h), c("high", "low"))
})

test_that("reference semantics modifies in place and returns invisibly", {

  dt <- data.table::data.table(
    g = factor(c("a", "b", "a"), levels = c("a", "b", "c")),
    h = c("low", "high", "low")
  )

  set_dt_copy_semantics("reference")
  dt_relevel_col(dt, levels = list(g = c("b", "a"), h = c("high", "low")))

  expect_identical(levels(dt$g), c("b", "a"))
  expect_identical(levels(dt$h), c("high", "low"))
})

test_that("idempotent when target levels already match for all columns", {

  dt <- data.table::data.table(
    g = factor(c("b", "a", "b"), levels = c("b", "a")),
    h = factor(c("high", "low", "high"), levels = c("high", "low"))
  )

  res <- dt_relevel_col(dt, levels = list(g = c("b", "a"), h = c("high", "low")))
  expect_identical(levels(res$g), c("b", "a"))
  expect_identical(levels(res$h), c("high", "low"))
  expect_identical(as.character(res$g), c("b", "a", "b"))
  expect_identical(as.character(res$h), c("high", "low", "high"))
})

test_that("errors if a named column in levels is missing in data", {

  dt <- data.table::data.table(g = factor(c("a", "b")))

  expect_error(
    dt_relevel_col(dt, levels = list(h = c("a", "b"))),
    "Column 'h' not found"
  )
})

#' Generate Core Safety Tables for Clinical Study Reports
#'
#' [AET01_table()] produces and combines the main safety summary tables
#' typically found in Section 14.3.1 of a Clinical Study Report (CSR). It
#' calculates patient counts and event totals for deaths, AE-related
#' withdrawals, total AEs, and adverse events of special interest (AESIs).
#'
#' @param adsl A subject-level dataset (typically ADaM ADSL).
#' @param adae A dataset of adverse events, preprocessed with AESI flags.
#' @param patient_var A string indicating the subject identifier variable (e.g., `"USUBJID"`).
#' @param treat_var A string indicating the treatment arm variable (e.g., `"ARM"`).
#' @param aesi_vars A character vector of binary AESI flags in `adae`.
#' @param aesi_heading Optional character string used as a heading in the AESI block.
#' @param indent A string used to indent AESI row labels (default is 2 spaces).
#'
#' @return A merged `data.table` summarising the main safety outcomes.
#'
#' @examples
#' AET01_summary <- AET01_table(
#'   adsl = adsl,
#'   adae = aesi,
#'   patient_var = "USUBJID",
#'   treat_var = "ARM",
#'   aesi_vars = c("FATAL", "SER", "SERWD", "SERDSM", "RELSER",
#'                 "WD", "DSM", "REL", "RELWD", "RELDSM", "SEV")
#' )
#' print(AET01_summary)
#'
#' @export
AET01_table <- function(adsl,
                        adae,
                        patient_var,
                        treat_var,
                        aesi_vars,
                        aesi_heading = "Total number of patients with at least one",
                        indent = "  ") {

  stopifnot(is.data.frame(adsl), is.data.frame(adae))
  missing_vars_adsl <- setdiff(c(patient_var, treat_var), names(adsl))
  missing_vars_adae <- setdiff(c(patient_var, treat_var), names(adae))

  if (length(missing_vars_adsl) > 0) {
    stop("The following variables are missing in `adsl`: ", toString(missing_vars_adsl))
  }

  if (length(missing_vars_adae) > 0) {
    stop("The following variables are missing in `adae`: ", toString(missing_vars_adae))
  }

  # Check AESI variables
  if (!is.character(aesi_vars) || length(aesi_vars) == 0) {
    stop("`aesi_vars` must be a non-empty character vector of variable names.")
  }

  missing_aesi_vars <- setdiff(aesi_vars, names(adae))
  if (length(missing_aesi_vars) > 0) {
    stop("The following `aesi_vars` are not found in `adae`: ", toString(missing_aesi_vars))
  }

  # --- Table Construction ---
  death_table <- event_count(
    dt = adsl,
    patient = patient_var,
    treat = treat_var,
    label = "Total number of deaths",
    .filters = "DTHFL == 'Y'"
  )

  withdrawal_table <- event_count(
    dt = adsl,
    patient = patient_var,
    treat = treat_var,
    label = "Total number of patients withdrawn from study due to an AE",
    .filters = "DCSREAS == 'ADVERSE EVENT'"
  )

  patients_with_ae_table <- event_count(
    dt = adae,
    patient = patient_var,
    treat = treat_var,
    label = "Total number of patients with at least one AE",
    .total_dt = adsl
  )

  total_ae_events_table <- total_events(
    dt = adae,
    treat = treat_var,
    label = "Total number of AEs"
  )

  aesi_table <- multi_event_true(
    dt = adae,
    event_vars = aesi_vars,
    patient = patient_var,
    treat = treat_var,
    heading = aesi_heading,
    .total_dt = adsl,
    indent = indent
  )

  merge_table_lists(list(
    patients_with_ae_table,
    total_ae_events_table,
    death_table,
    withdrawal_table,
    aesi_table
  ))
}

test_that("Input validation of AET01 table creation works as expected", {
  expect_error(
    AET01_table(
      adsl = adsl,
      adae = aesi,
      patient_var = "USUBJID1",
      treat_var = "TRT09A",
      aesi_vars = c("FATAL", "SER")
    ),
    "The following variables are missing in `adsl`: USUBJID1, TRT09A"
  )

  expect_error(
    AET01_table(
      adsl = adsl,
      adae = iris,
      patient_var = "USUBJID",
      treat_var = "TRT01A",
      aesi_vars = c("FATAL", "SER")
    ),
    "The following variables are missing in `adae`: USUBJID, TRT01A"
  )

  expect_error(
    AET01_table(
      adsl = adsl,
      adae = aesi,
      patient_var = "USUBJID",
      treat_var = "TRT01A",
      aesi_vars = NULL
    ),
    "`aesi_vars` must be a non-empty character vector of variable names."
  )

  expect_error(
    AET01_table(
      adsl = adsl,
      adae = aesi,
      patient_var = "USUBJID",
      treat_var = "TRT01A",
      aesi_vars = c("FATALLY", "SER")
    ),
    "The following `aesi_vars` are not found in `adae`: FATALLY"
  )
})

test_that("Able to re-create the AET01 table using data.table", {
  AET01_summary <- AET01_table(
    adsl = adsl,
    adae = aesi,
    patient_var = "USUBJID",
    treat_var = "ARM",
    aesi_vars = c(
      "FATAL", "SER", "SERWD", "SERDSM", "RELSER", "WD", "DSM", "REL", "RELWD", "RELDSM", "SEV"
    )
  )

  expect_s3_class(AET01_summary, "data.table")
  expect_named(AET01_summary, c("stats", levels(adsl$ARM)))
  expect_identical(
    AET01_summary[stats == "Total number of AEs", `A: Drug X`],
    aesi[ARM == "A: Drug X", as.character(.N)]
  )

  expect_identical(
    AET01_summary[endsWith(stats, "Related AE"), `C: Combination`],
    aesi[
      ARM == "C: Combination" & REL == TRUE,
      paste(
        data.table::uniqueN(USUBJID),
        sprintf("(%.1f%%)", data.table::uniqueN(USUBJID) / sum(adsl$ARM == "C: Combination") * 100)
      )
    ]
  )
})

is_pkg_available <- function(pkg) {
  (requireNamespace(pkg, quietly = TRUE))
}

assert_pkg_is_installed <- function(pkg) {
  if (isFALSE(is_pkg_available(pkg = pkg))) {
    stop("Package ", pkg, " must be installed to use this function.", call. = FALSE)
  }
}

#' Get or set data.table copy semantics
#'
#' These functions control how [maybe_copy_dt()] decides whether to
#' return a `data.table` by reference (in place) or by value (as a deep copy).
#'
#' @details
#' The copy semantics are stored in the global option
#' `dtlg_dt_copy_semantics`. The option can take two values:
#'
#' - `"reference"` (default): inputs are treated with reference semantics.
#'   - If the input is already a `data.table`, it is returned unchanged and
#'     aliases are preserved.
#'   - If the input is a `data.frame`, it is converted to a `data.table`
#'     in place via [data.table::setDT()], mutating the caller’s object.
#'
#' - `"value"`: inputs are treated with value semantics.
#'   - The input is converted to a `data.table` (if necessary) and a deep
#'     copy is returned, leaving the original unchanged.
#'
#' @returns
#' - `dt_copy_semantics()` returns the current semantics as a string,
#'   `"reference"` or `"value"`.
#'
#' - `set_dt_copy_semantics()` sets the semantics, returning the previous
#'   semantics invisibly.
#'
#' @param dt_copy_semantics Character string. Either `"reference"` or `"value"`.
#'
#' @examples
#' # Get current semantics (defaults to "reference")
#' dt_copy_semantics()
#'
#' # Switch to value semantics
#' old <- set_dt_copy_semantics("value")
#' dt_copy_semantics()
#'
#' # Restore previous semantics
#' set_dt_copy_semantics(old)
#'
#' @seealso [maybe_copy_dt()]
#'
#' @export
dt_copy_semantics <- function() {
  getOption(x = "dtlg_dt_copy_semantics", default = "reference")
}

#' @rdname dt_copy_semantics
#' @export
set_dt_copy_semantics <- function(dt_copy_semantics = c("reference", "value")) {
  dt_copy_semantics <- match.arg(dt_copy_semantics)
  old_semantics <- dt_copy_semantics()
  options(dtlg_dt_copy_semantics = dt_copy_semantics)
  invisible(old_semantics)
}

#' Return a data.table by reference or by value
#'
#' [maybe_copy_dt()] returns its input as a `data.table`, with behaviour
#' controlled by the global copy semantics option [dt_copy_semantics()].
#'
#' - If the semantics are `"reference"` (default):
#'   * If `x` is already a `data.table`, it is returned unchanged. Aliasing
#'     holds, so mutations with `:=` will affect both input and output.
#'
#'   * If `x` is a `data.frame`, it is converted to a `data.table` in place
#'     via [data.table::setDT()], mutating the caller’s object. The returned
#'     object is a `data.table` with the same contents. For efficiency, the
#'     column vectors are reused without a deep copy.
#'
#' - If the semantics are `"value"`:
#'   * `x` is converted to a `data.table` (if necessary) and a deep copy is
#'     returned. Mutating the result does not affect the input.
#'
#' @param x A `data.table` or `data.frame`.
#'
#' @returns
#' A `data.table`. Whether the return value aliases the input depends on
#' the semantics:
#'
#' - `"reference"`: input is mutated in place, aliasing guaranteed if
#'   `x` is already a `data.table`.
#' - `"value"`: a fresh copy is returned, independent of the input.
#'
#' @seealso [dt_copy_semantics()], [set_dt_copy_semantics()]
#'
#' @examples
#' # Default: reference semantics
#' df <- data.frame(a = 1:3)
#' out <- maybe_copy_dt(df)
#' data.table::is.data.table(df) # TRUE, converted in place
#'
#' # Switch to value semantics
#' old <- set_dt_copy_semantics("value")
#' dt <- data.table::data.table(a = 1:3)
#' out2 <- maybe_copy_dt(dt)
#' out2[, b := 99L]
#' "b" %in% names(dt)  # FALSE, original unchanged
#'
#' # Restore previous semantics
#' set_dt_copy_semantics(old)
#'
#' @export
maybe_copy_dt <- function(x) {
  error_msg <- paste("Unknown data.table copy semantics option:",
                     "must be 'reference' or 'value'")

  sem <- dt_copy_semantics()

  if (identical(sem, "reference")) {
    data.table::setDT(x)       # mutates in place
    return(x)                  # return the very same object
  }

  if (identical(sem, "value")) {
    return(data.table::copy(data.table::as.data.table(x)))
  }

  stop(error_msg, call. = FALSE)
}

test_that("default semantics is 'reference'", {
  withr::local_options(list(dtlg_dt_copy_semantics = NULL))
  expect_identical(dt_copy_semantics(), "reference")
})

test_that("option state does not leak", {
  withr::local_options(list(dtlg_dt_copy_semantics = "reference"))
  expect_identical(dt_copy_semantics(), "reference")
})


test_that("set_dt_copy_semantics() validates and returns previous value",
          {
            withr::with_options(list(dtlg_dt_copy_semantics = "reference"), {
              old <- set_dt_copy_semantics("value")
              expect_identical(old, "reference")
              expect_identical(dt_copy_semantics(), "value")

              old2 <- set_dt_copy_semantics("reference")
              expect_identical(old2, "value")
              expect_identical(dt_copy_semantics(), "reference")

              expect_error(set_dt_copy_semantics("bogus"))
            })
          })

test_that("maybe_copy_dt() returns a data.table in both modes", {
  df1 <- data.frame(a = 1:3)
  df2 <- data.frame(a = 1:3)

  withr::with_options(list(dtlg_dt_copy_semantics = "reference"), {
    out_ref <- maybe_copy_dt(df1)
    expect_true(data.table::is.data.table(df1))
    expect_true(data.table::is.data.table(out_ref))
  })

  withr::with_options(list(dtlg_dt_copy_semantics = "value"), {
    out_val <- maybe_copy_dt(df2)
    expect_false(data.table::is.data.table(df2))
    expect_true(data.table::is.data.table(out_val))
  })
})

test_that("reference semantics: in-place mutation holds", {
  withr::with_options(list(dtlg_dt_copy_semantics = "reference"), {
    # Case 1: input is already a data.table
    x_dt <- data.table::data.table(a = 1:3)
    y_dt <- maybe_copy_dt(x_dt)
    y_dt[, b := 99L]
    expect_true("b" %in% names(x_dt))
    expect_identical(x_dt$b, rep.int(99L, 3))

    # Case 2: input is a data.frame
    x_df <- data.frame(a = 1:3)
    y2 <- maybe_copy_dt(x_df)
    expect_true(data.table::is.data.table(x_df))   # original mutated in place
    expect_equal(x_df, y2)                         # contents identical
    # but DO NOT test aliasing between x_df and y2
  })
})

test_that("maybe_copy_dt avoids deep copy for regular (non-ALTREP) columns", {
  # Use explicitly materialised vectors (not 1:3)
  df <- data.frame(
    a = as.integer(c(1L, 2L, 3L)),
    b = c("x1", "x2", "x3"),
    stringsAsFactors = FALSE
  )

  addr_a_before <- data.table::address(df$a)
  addr_b_before <- data.table::address(df$b)

  df2 <- maybe_copy_dt(df)

  expect_true(data.table::is.data.table(df2))
  expect_identical(data.table::address(df2$a), addr_a_before)
  expect_identical(data.table::address(df2$b), addr_b_before)
})

test_that("reference semantics: data.table input aliases (:= propagates)", {
  withr::local_options(list(dtlg_dt_copy_semantics = "reference"))
  x <- data.table::data.table(a = 1:3)
  y <- maybe_copy_dt(x)
  y[, b := 9L]
  expect_true("b" %in% names(x))
  expect_identical(x$b, rep.int(9L, 3))
})

test_that("reference semantics: data.frame converts in place (no deep copy of cols)", {
  withr::local_options(list(dtlg_dt_copy_semantics = "reference"))
  df <- data.frame(
    a = as.integer(c(1L, 2L, 3L)),
    b = c("x1", "x2", "x3"),
    stringsAsFactors = FALSE
  )
  a_addr <- data.table::address(df$a)
  b_addr <- data.table::address(df$b)

  res <- maybe_copy_dt(df)

  expect_true(data.table::is.data.table(df))   # caller mutated
  expect_true(data.table::is.data.table(res))  # return is DT
  # same column vectors reused (no deep copy)
  expect_identical(data.table::address(df$a), a_addr)
  expect_identical(data.table::address(df$b), b_addr)
})

test_that("value semantics: always a fresh copy (independent of input type)", {
  withr::local_options(list(dtlg_dt_copy_semantics = "value"))

  # data.table input
  dt <- data.table::data.table(a = 1:3)
  out1 <- maybe_copy_dt(dt)
  expect_true(data.table::is.data.table(out1))
  out1[, b := 1L]
  expect_false("b" %in% names(dt))

  # data.frame input
  df <- data.frame(a = 1:3)
  out2 <- maybe_copy_dt(df)
  expect_true(data.table::is.data.table(out2))
  expect_false(data.table::is.data.table(df))
})

test_that("value semantics: DT input returns a deep copy (pointer differs)", {
  withr::local_options(list(dtlg_dt_copy_semantics = "value"))
  x <- data.table::data.table(a = 1:3)
  addr_before <- data.table::address(x)
  y <- maybe_copy_dt(x)
  expect_false(identical(data.table::address(y), addr_before))
})

test_that("setter/getter semantics lifecycle", {
  withr::local_options(list(dtlg_dt_copy_semantics = "reference"))
  old <- set_dt_copy_semantics("value")
  expect_identical(old, "reference")
  expect_identical(dt_copy_semantics(), "value")
  old2 <- set_dt_copy_semantics("reference")
  expect_identical(old2, "value")
  expect_identical(dt_copy_semantics(), "reference")
  expect_error(set_dt_copy_semantics("bogus"))
})

test_that("invalid option triggers clear error", {
  withr::local_options(list(dtlg_dt_copy_semantics = "neither"))
  expect_error(
    maybe_copy_dt(data.frame(a = 1)),
    "Unknown data.table copy semantics option:.*'reference' or 'value'"
  )
})

test_that("inline expression in reference mode returns DT but cannot mutate caller", {
  withr::local_options(list(dtlg_dt_copy_semantics = "reference"))
  res <- maybe_copy_dt(data.frame(a = 1:3))
  expect_true(data.table::is.data.table(res))
  # no caller symbol exists; just document this behaviour (no expect here)
})

test_that("reference semantics: DT input keeps same pointer", {
  withr::local_options(list(dtlg_dt_copy_semantics = "reference"))
  x <- data.table::data.table(a = 1:3)
  addr_before <- data.table::address(x)
  y <- maybe_copy_dt(x)
  expect_identical(data.table::address(y), addr_before)
})

test_that("idempotency across repeated calls", {
  # reference
  withr::local_options(list(dtlg_dt_copy_semantics = "reference"))
  x <- data.table::data.table(a = 1:3)
  y1 <- maybe_copy_dt(x)
  y2 <- maybe_copy_dt(y1)
  expect_identical(data.table::address(y2), data.table::address(x))

  # value
  withr::local_options(list(dtlg_dt_copy_semantics = "value"))
  x2 <- data.table::data.table(a = 1:3)
  yv1 <- maybe_copy_dt(x2)
  yv2 <- maybe_copy_dt(yv1)
  # second call still returns a fresh copy
  expect_false(identical(data.table::address(yv2), data.table::address(yv1)))
})

test_that("zero-row inputs behave consistently", {
  withr::local_options(list(dtlg_dt_copy_semantics = "reference"))
  df0 <- data.frame(a = integer())
  out_ref <- maybe_copy_dt(df0)
  expect_true(data.table::is.data.table(df0))
  expect_true(data.table::is.data.table(out_ref))
  expect_identical(nrow(out_ref), 0L)

  withr::local_options(list(dtlg_dt_copy_semantics = "value"))
  df0b <- data.frame(a = integer())
  out_val <- maybe_copy_dt(df0b)
  expect_false(data.table::is.data.table(df0b))
  expect_true(data.table::is.data.table(out_val))
  expect_identical(nrow(out_val), 0L)
})

test_that("class and names are preserved/expected", {
  withr::local_options(list(dtlg_dt_copy_semantics = "reference"))
  df <- structure(data.frame(a = 1:3, b = 4:6), my_attr = 1L)
  out <- maybe_copy_dt(df)
  expect_identical(class(out), c("data.table", "data.frame"))
  expect_identical(names(out), c("a", "b"))
})

test_that("reference semantics with inline expr: returns DT, no external mutation", {
  withr::local_options(list(dtlg_dt_copy_semantics = "reference"))
  res <- maybe_copy_dt(data.frame(a = 1:3))
  expect_true(data.table::is.data.table(res))
})

is_bool <- function(x) {
  is.logical(x) && identical(length(x), 1L) && !is.na(x)
}

is_scalar_number <- function(x) {
  is.numeric(x) && identical(length(x), 1L) && !is.na(x)
}

is_string <- function(x) {
  is.character(x) && identical(length(x), 1L) && !is.na(x)
}

and <- function(...) {
  dots <- list(...)
  Reduce(`&`, dots, init = TRUE)
}

#' Subject-Level Analysis Dataset (ADSL) example dataset
#'
#' [adsl] is a re-export of the [random.cdisc.data::cadsl] dataset, included in
#' `{dtlg}` for function usage illustration and testing.
#'
#' @inherit random.cdisc.data::cadsl format
#'
#' @examples
#' adsl
#'
"adsl"

#' ADaM Basic Data Structure (BDS) example dataset
#'
#' [adlb] is a re-export of the [random.cdisc.data::cadlb] dataset, included in
#' `{dtlg}` for function usage illustration and testing.
#'
#' @inherit random.cdisc.data::cadlb format
#'
#' @examples
#' adlb
#'
"adlb"

#' Adverse Event Analysis Dataset example dataset
#'
#' [adae] is a re-export of the [random.cdisc.data::cadae] dataset, included in
#' `{dtlg}` for function usage illustration and testing.
#'
#' @inherit random.cdisc.data::adae format
#'
#' @examples
#' adae
#'
"adae"

#' Adverse Events of Special Interest (AESI) example dataset
#'
#' [aesi] is a modified version of the [random.cdisc.data::cadae] dataset,
#' filtered to include only analysis-flagged records (`ANL01FL == "Y"`) and
#' extended with binary indicator variables corresponding to adverse events of
#' special interest (AESIs).
#'
#' These derived flags include seriousness, severity, fatality, relatedness, and
#' treatment consequence (e.g., dose modification or withdrawal), and are used to
#' illustrate key safety summaries in clinical reporting.
#'
#' Each derived variable is labeled using [dtlg::with_label()] for compatibility
#' with tabulation functions.
#'
#' This dataset is included in `{dtlg}` to support function testing, usage
#' examples, and reproducible safety analyses.
#'
#' @format A `data.frame` with a subset of rows from `cadae` and additional
#' derived columns including:
#' \describe{
#'   \item{FATAL}{Logical flag for fatal AEs (`AESDTH == "Y"`).}
#'   \item{SEV}{Logical flag for severe AEs (`AESEV == "SEVERE"`).}
#'   \item{SER}{Logical flag for serious AEs (`AESER == "Y"`).}
#'   \item{SERWD}{Serious AE leading to withdrawal (`AESER == "Y" & AEACN == "DRUG WITHDRAWN"`).}
#'   \item{SERDSM}{Serious AE leading to dose modification/interruption.}
#'   \item{RELSER}{Serious and related AE.}
#'   \item{WD}{AE leading to withdrawal.}
#'   \item{DSM}{AE leading to dose modification/interruption.}
#'   \item{REL}{Related AE.}
#'   \item{RELWD}{Related AE leading to withdrawal.}
#'   \item{RELDSM}{Related AE leading to dose modification/interruption.}
#' }
#'
#' @seealso [random.cdisc.data::cadae], [dtlg::multi_event_true()]
#'
#' @examples
#' aesi
#'
"aesi"

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary-table.R
\name{summary_table}
\alias{summary_table}
\title{Summary Table}
\usage{
summary_table(
  dt,
  target,
  treat,
  target_name = target,
  indent = nbsp(n = 4L),
  .total_dt = dt,
  pct_dec = 1,
  treat_order = NULL,
  skip_absent = TRUE,
  inc_missing = TRUE
)
}
\arguments{
\item{dt}{A \code{data.frame} containing, at least, the variables indicated in
\code{target} and \code{treat}.}

\item{target}{Target variable passed as a string for which summary
statistics are to be calculated.}

\item{treat}{A string indicating the grouping variable, e.g. the variable
specifying the treatment population.}

\item{target_name}{Heading for the target variable as a string. Defaults to
\code{target}.}

\item{indent}{A string to be used as indentation of summary statistics
labels. Defaults to four HTML non-breaking spaces (\verb{&nbsp;}).}

\item{.total_dt}{Separate table from \code{dt} from which to derive total counts
per group.}

\item{pct_dec}{Decimal places for reported figures.}

\item{treat_order}{Customise the column order of the output table.}

\item{skip_absent}{Whether to ignore variables passed in \code{treat_order} that
are absent from \code{dt}. Default is \code{TRUE}; \code{FALSE} will throw an error in
case there are missing variables.}

\item{inc_missing}{If any of the \code{target} variables are numeric, then used
as a toggle to determine whether or not "Missing" appears in the summary stats:
\describe{
\item{\code{TRUE}}{(default) The Missing row is always displayed}
\item{\code{NA}}{The Missing row is only displayed if any missing values are present}
\item{\code{FALSE}}{The Missing row is never included in the table}
}}
}
\value{
A \code{data.table} of summary statistics. The format depends on the
type of the \code{target} variable:
\itemize{
\item If the \code{target} variable is categorical, i.e. type \code{character}, \code{factor}
or \code{logical} then the output is that of \code{\link[=calc_counts]{calc_counts()}}.
\item If the \code{target} variable is numeric, then the output is that of
\code{\link[=calc_desc]{calc_desc()}}.
}
}
\description{
\code{\link[=summary_table]{summary_table()}} summarises clinical variables into a report table using
\code{data.table} as backend.
}
\examples{
dmg_vars <- c("AGE", "RACE", "ETHNIC")
dmg_var_lbls <- c("Age (yr)", "Race", "Ethnicity")

# Demographics table (DMT01)
summary_table(
  adsl,
  target = dmg_vars,
  treat = 'ARM',
  target_name = dmg_var_lbls
)

# Demographics table (DMT01) with continuous variable (e.g., BMRKR1)
summary_table(
  adsl,
  target = c(dmg_vars, "BMRKR1"),
  treat = 'ARM',
  target_name = c(dmg_var_lbls, "Biomarker 1")
)

}
\seealso{
\code{\link[=tern_summary_table]{tern_summary_table()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dtlg-package.R
\docType{package}
\name{dtlg-package}
\alias{dtlg}
\alias{dtlg-package}
\title{dtlg: A Performance-Focused Package for Clinical Trial Tables}
\description{
Create high-performance clinical reporting tables (TLGs) from ADaM-like inputs. The package provides a consistent, programmatic API to generate common tables such as demographics, adverse event incidence, and laboratory summaries, using 'data.table' for fast aggregation over large populations. Functions support flexible target-variable selection, stratification by treatment, and customizable summary statistics, and return tidy, machine-readable results ready to render with downstream table/formatting packages in analysis pipelines.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://AscentSoftware.github.io/dtlg/}
  \item Report bugs at \url{https://github.com/AscentSoftware/dtlg/issues}
}

}
\author{
\strong{Maintainer}: Ramiro Magno \email{ramiro.morgado@acuityanalytics.com}

Authors:
\itemize{
  \item Max Ebenezer-Brown \email{max.ebenezer-brown@acuityanalytics.com}
  \item Max Norman
  \item Xinye Li \email{xinye.li@acuityanalytics.com}
  \item Anja Peebles-Brown \email{anna.peebles-brown@acuityanalytics.com}
  \item Ashley Baldry \email{ashley.baldry@acuityanalytics.com}
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/events.R
\name{event_count}
\alias{event_count}
\title{Count events}
\usage{
event_count(
  dt,
  patient,
  treat,
  label,
  .filters = NULL,
  .total_dt = dt,
  pct_dec = 1
)
}
\arguments{
\item{dt}{A \code{data.frame} containing, at least, the variables indicated in
\code{target} and \code{treat}.}

\item{patient}{A string indicating the subject identifying variable.}

\item{treat}{A string indicating the grouping variable, e.g. the variable
specifying the treatment population.}

\item{label}{A string to be used as label in the output reporting table. This
should be a text descriptive of the event being counted.}

\item{.filters}{Predicate expressions identifying events in \code{dt}. Argument
should be passed as a \code{character} vector of expressions to be evaluated in
the frame of \code{dt}.}

\item{.total_dt}{Separate table from \code{dt} from which to derive total counts
per group.}

\item{pct_dec}{This argument is ignored, and is only kept for backward
compatibility reasons.}
}
\value{
A one-element list, where the element is a \code{data.table}.
}
\description{
\code{\link[=event_count]{event_count()}} counts events defined by predicate expressions passed in
\code{.filters}.
}
\examples{
# Count deaths per arm.
event_count(
  adsl,
  patient = "USUBJID",
  treat = "ARM",
  label = "Total number of deaths",
  .filters = "DTHFL == 'Y'"
)[[1]]

# Count patients withdraw from study due to an adverse event.
withdrawn_lbl <- "Total number of patients withdrawn from study due to an AE"
event_count(
  adsl,
  patient = "USUBJID",
  treat = "ARM",
  label = withdrawn_lbl,
  .filters = "DCSREAS == 'ADVERSE EVENT'"
)[[1]]

# Count patients with at least one adverse event.
# NB: When `.filters` is `NULL` (i.e., omitted), all records in `dt` are used
# for counting events.
event_count(
  adae,
  patient = "USUBJID",
  treat = "ARM",
  label = "Total number of patients with at least one AE",
  .filters = "ANL01FL == 'Y'",
  .total_dt = adsl
)[[1]]

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dt-copy-semantics.R
\name{dt_copy_semantics}
\alias{dt_copy_semantics}
\alias{set_dt_copy_semantics}
\title{Get or set data.table copy semantics}
\usage{
dt_copy_semantics()

set_dt_copy_semantics(dt_copy_semantics = c("reference", "value"))
}
\arguments{
\item{dt_copy_semantics}{Character string. Either \code{"reference"} or \code{"value"}.}
}
\value{
\itemize{
\item \code{dt_copy_semantics()} returns the current semantics as a string,
\code{"reference"} or \code{"value"}.
\item \code{set_dt_copy_semantics()} sets the semantics, returning the previous
semantics invisibly.
}
}
\description{
These functions control how \code{\link[=maybe_copy_dt]{maybe_copy_dt()}} decides whether to
return a \code{data.table} by reference (in place) or by value (as a deep copy).
}
\details{
The copy semantics are stored in the global option
\code{dtlg_dt_copy_semantics}. The option can take two values:
\itemize{
\item \code{"reference"} (default): inputs are treated with reference semantics.
\itemize{
\item If the input is already a \code{data.table}, it is returned unchanged and
aliases are preserved.
\item If the input is a \code{data.frame}, it is converted to a \code{data.table}
in place via \code{\link[data.table:setDT]{data.table::setDT()}}, mutating the caller’s object.
}
\item \code{"value"}: inputs are treated with value semantics.
\itemize{
\item The input is converted to a \code{data.table} (if necessary) and a deep
copy is returned, leaving the original unchanged.
}
}
}
\examples{
# Get current semantics (defaults to "reference")
dt_copy_semantics()

# Switch to value semantics
old <- set_dt_copy_semantics("value")
dt_copy_semantics()

# Restore previous semantics
set_dt_copy_semantics(old)

}
\seealso{
\code{\link[=maybe_copy_dt]{maybe_copy_dt()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/with_label.R
\name{with_label}
\alias{with_label}
\title{Add a label attribute to an object}
\usage{
with_label(x, label)
}
\arguments{
\item{x}{An R object.}

\item{label}{A label provided as a single string.}
}
\value{
\code{x} labeled by \code{label}.
}
\description{
Add a label attribute to an object
}
\examples{
label(1)
label(with_label(1, "my label"))

}
\seealso{
\code{\link[=label]{label()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/indentation.R
\name{nbsp}
\alias{nbsp}
\title{Generate Non-Breaking Spaces for HTML Output}
\usage{
nbsp(n = 1L)
}
\arguments{
\item{n}{Number of non-breaking spaces. Defaults to \code{1}.}
}
\value{
A character string containing \code{n} HTML non-breaking spaces
(\verb{&nbsp;}).
}
\description{
\code{\link[=nbsp]{nbsp()}} generates a string of HTML non-breaking spaces (\verb{&nbsp;}).
}
\examples{
# One non-breaking space
nbsp()
nbsp(1)

# Several non-breaking spaces
nbsp(n = 2)
nbsp(n = 5)

# When `n` is zero, an empty string is returned.
nbsp(n = 0)

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print_dtlg.R
\name{print_dtlg}
\alias{print_dtlg}
\title{Print a \code{dtlg} table}
\usage{
print_dtlg(
  dt,
  row.names = FALSE,
  trunc.cols = TRUE,
  class = FALSE,
  nrows = Inf,
  justify = "left"
)
}
\arguments{
\item{dt}{A \code{dtlg} table, typically a \code{data.frame} or \code{data.table}.}

\item{row.names}{ If \code{TRUE}, row indices will be printed alongside \code{x}. }

\item{trunc.cols}{ If \code{TRUE}, only the columns that can be printed in the console without wrapping the columns to new lines will be printed (similar to \code{tibbles}). }

\item{class}{ If \code{TRUE}, the resulting output will include above each column its storage class (or a self-evident abbreviation thereof). When combined with \code{col.names="auto"} and tables >20 rows, classes will also appear at the bottom.}

\item{nrows}{ The number of rows which will be printed before truncation is enforced. }

\item{justify}{String. Column alignment; one of \code{"left"}, \code{"right"},
\code{"centre"}, or \code{"none"}. Defaults to \code{"left"}.}
}
\value{
Invisibly returns the printed object.
}
\description{
A convenience wrapper around \code{\link[=print]{print()}} for printing \code{dtlg} tables with
consistent formatting options.
}
\examples{
calc_stats(dt = adsl, "AGE", treat = "ARM", indent = "  ")[[1]] |>
  print_dtlg()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dt-copy-semantics.R
\name{maybe_copy_dt}
\alias{maybe_copy_dt}
\title{Return a data.table by reference or by value}
\usage{
maybe_copy_dt(x)
}
\arguments{
\item{x}{A \code{data.table} or \code{data.frame}.}
}
\value{
A \code{data.table}. Whether the return value aliases the input depends on
the semantics:
\itemize{
\item \code{"reference"}: input is mutated in place, aliasing guaranteed if
\code{x} is already a \code{data.table}.
\item \code{"value"}: a fresh copy is returned, independent of the input.
}
}
\description{
\code{\link[=maybe_copy_dt]{maybe_copy_dt()}} returns its input as a \code{data.table}, with behaviour
controlled by the global copy semantics option \code{\link[=dt_copy_semantics]{dt_copy_semantics()}}.
}
\details{
\itemize{
\item If the semantics are \code{"reference"} (default):
\itemize{
\item If \code{x} is already a \code{data.table}, it is returned unchanged. Aliasing
holds, so mutations with \verb{:=} will affect both input and output.
\item If \code{x} is a \code{data.frame}, it is converted to a \code{data.table} in place
via \code{\link[data.table:setDT]{data.table::setDT()}}, mutating the caller’s object. The returned
object is a \code{data.table} with the same contents. For efficiency, the
column vectors are reused without a deep copy.
}
\item If the semantics are \code{"value"}:
\itemize{
\item \code{x} is converted to a \code{data.table} (if necessary) and a deep copy is
returned. Mutating the result does not affect the input.
}
}
}
\examples{
# Default: reference semantics
df <- data.frame(a = 1:3)
out <- maybe_copy_dt(df)
data.table::is.data.table(df) # TRUE, converted in place

# Switch to value semantics
old <- set_dt_copy_semantics("value")
dt <- data.table::data.table(a = 1:3)
out2 <- maybe_copy_dt(dt)
out2[, b := 99L]
"b" \%in\% names(dt)  # FALSE, original unchanged

# Restore previous semantics
set_dt_copy_semantics(old)

}
\seealso{
\code{\link[=dt_copy_semantics]{dt_copy_semantics()}}, \code{\link[=set_dt_copy_semantics]{set_dt_copy_semantics()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/round.R
\name{round_sum}
\alias{round_sum}
\title{Rounds numbers while preserving the total sum}
\usage{
round_sum(x, digits = 0L)
}
\arguments{
\item{x}{A numeric vector of non-negative values that you want to round.
Missing values (\code{NA}) are ignored.}

\item{digits}{The number of decimal places to round to. Default is \code{0}
(integer rounding).}
}
\value{
A numeric vector of the same length as \code{x}, with values rounded in
such a way that the total sum is preserved.
}
\description{
\code{\link[=round_sum]{round_sum()}} rounds a numeric vector of non-negative values to a specified
number of decimal places while ensuring that the sum of the rounded value
remains as close as possible to the original total.
}
\examples{
# Rounds to integers, preserving the sum of 100.
x <- c(33.3333, 33.3333, 33.3334)
(y <- round_sum(x))
identical(sum(x), sum(y))

# Rounds to integers, preserving the sum of 1002.
x <- c(100.5, 200.25, 300.75, 400.5)
(y <- round_sum(x))
identical(sum(x), sum(y))

# Rounds to one decimal place, preserving the total sum.
x <- c(12.345, 67.890, 19.765)
(y <- round_sum(x))
identical(sum(x), sum(y))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tern.R
\name{as_dtlg_table}
\alias{as_dtlg_table}
\title{Convert a TableTree to a dtlg table}
\usage{
as_dtlg_table(tt, .label_col = "stats")
}
\arguments{
\item{tt}{A TableTree object. Typically obtained with \code{\link[=tern_summary_table]{tern_summary_table()}}.}

\item{.label_col}{Label for stats' column.}
}
\value{
A data.table.
}
\description{
\code{\link[=as_dtlg_table]{as_dtlg_table()}} reformats a TableTree object into a format close to that
of dtlg's \code{data.table}.
}
\examples{
vars <- c('AGE', 'RACE', 'ETHNIC', 'BMRKR1')
var_labels <- c("Age (yr)", "Race", "Ethnicity", "Continuous Level Biomarker 1")

# Summary statistics table split by ARM with custom labels.
(tt <- tern_summary_table(
  adsl,
  target = vars,
  treat = 'ARM',
  target_name = var_labels
))

# Format as a dtlg table
as_dtlg_table(tt)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calc-stats.R
\name{calc_counts}
\alias{calc_counts}
\title{Calculate counts of a categorical variable}
\usage{
calc_counts(
  dt,
  target,
  target_name = target,
  treat,
  indent = nbsp(n = 4L),
  .total_dt = NULL,
  pct_dec = 1
)
}
\arguments{
\item{dt}{A \code{data.frame} containing, at least, the variables indicated in
\code{target} and \code{treat}.}

\item{target}{Target variable passed as a string for which summary
statistics are to be calculated.}

\item{target_name}{Heading for the target variable as a string. Defaults to
\code{target}.}

\item{treat}{A string indicating the grouping variable, e.g. the variable
specifying the treatment population.}

\item{indent}{A string to be used as indentation of summary statistics
labels. Defaults to four HTML non-breaking spaces (\verb{&nbsp;}).}

\item{.total_dt}{Separate table from \code{dt} from which to derive total counts
per group.}

\item{pct_dec}{This argument is ignored, and is only kept for backward
compatibility reasons.}
}
\value{
A list containing a \code{data.table} formatted as follows:
\itemize{
\item First column is named \code{stats} and contains the target variable name
indicated in \code{target} in the first row. Subsequent rows contain the
levels of \code{target}.
\item Other columns are for the levels of the grouping variable (\code{treat}).
\item All columns are of character type.
}

This table is structured for easy integration with Shiny output widgets.
}
\description{
\code{\link[=calc_counts]{calc_counts()}} counts observations of a categorical variable (\code{target}) by
another (\code{treat}) and reports summary statistics in clinical trial reporting
format.
}
\examples{
calc_counts(dt = adsl, "RACE", treat = "ARM", indent = "  ")[[1]]

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tern.R
\name{tern_AET02_table}
\alias{tern_AET02_table}
\title{Generate AET02-style AE summary using \code{tern} and \code{rtables}}
\usage{
tern_AET02_table(
  adsl,
  adae,
  patient,
  treat,
  target = "AEDECOD",
  rows_by = "AEBODSYS",
  indent = "  "
)
}
\arguments{
\item{adsl}{Subject-level dataset.}

\item{adae}{Adverse event dataset.}

\item{patient}{Unique subject identifier variable.}

\item{treat}{Treatment arm variable.}

\item{target}{Preferred term variable (default: \code{"AEDECOD"}).}

\item{rows_by}{Higher-level nesting term (default: \code{"AEBODSYS"}).}

\item{indent}{Ignored (included for compatibility).}
}
\value{
A \code{TableTree} object with AE summary by SOC/PT.
}
\description{
This function builds a System Organ Class (SOC) and Preferred Term (PT)
adverse event summary table, following the AET02 CSR format, using the
\code{tern} and \code{rtables} packages.
}
\seealso{
\code{\link[=AET02_table]{AET02_table()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AET02_table.R
\name{AET02_table}
\alias{AET02_table}
\title{Create AET02-style AE summary table}
\usage{
AET02_table(
  adsl,
  adae,
  patient,
  treat,
  target = "AEDECOD",
  rows_by = "AEBODSYS",
  indent = nbsp(n = 4L)
)
}
\arguments{
\item{adsl}{Subject-level dataset.}

\item{adae}{Adverse event dataset.}

\item{patient}{Unique subject identifier variable.}

\item{treat}{Treatment arm variable.}

\item{target}{Preferred term variable for grouping (default: \code{"AEDECOD"}).}

\item{rows_by}{Higher-level term for nesting (default: \code{"AEBODSYS"}).}

\item{indent}{Character or string to indent nested rows (default: 4 non-breaking spaces).}
}
\value{
A merged data.table containing AE summary.
}
\description{
Summarises adverse events in a format similar to the AET02 table from a CSR,
showing total AE counts, patients with AEs, and a breakdown by System Organ
Class (SOC) and Preferred Term (PT).
}
\examples{
# Create a AET02 table
AET02_table(
  adsl = adsl,
  adae = aesi,
  patient = "USUBJID",
  treat = "ARM",
  target = "AEDECOD",
  rows_by = "AEBODSYS",
  indent = "  "
)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary-table.R
\name{summary_table_by_targets}
\alias{summary_table_by_targets}
\title{Create a summary table using multiple rows for grouping on two target column
ideal for creating change from baseline tables}
\usage{
summary_table_by_targets(
  dt,
  target,
  treat,
  rows_by,
  indent = nbsp(n = 4L),
  .total_dt = NULL,
  pct_dec = 1,
  treat_order = NULL,
  skip_absent = TRUE,
  sep = "."
)
}
\arguments{
\item{dt}{table to perform function on}

\item{target}{vector of column names desired to obtain information on}

\item{treat}{string of treatment variable used for splitting / grouping data}

\item{rows_by}{string, grouping variable to split events by.}

\item{indent}{indent to be used for display and formatting purposes}

\item{.total_dt}{optional table for total counts to be derived}

\item{pct_dec}{decimal places for percentages}

\item{treat_order}{customise the column order of output table}

\item{skip_absent}{Logical, default \code{TRUE}. Passed to
\code{data.table::setcolorder}, if \code{treat_order} includes columns not present in
\code{dt}:
\itemize{
\item \code{TRUE} will silently ignore them
\item \code{FALSE} will throw an error
}}

\item{sep}{character string to separate the terms}
}
\value{
data.table
}
\description{
Create a summary table using multiple rows for grouping on two target column
ideal for creating change from baseline tables
}
\examples{
adlb <- random.cdisc.data::cadlb |>
  dplyr::filter(AVISIT != "SCREENING")

labs <- summary_table_by_targets(
  dt = adlb,
  target = c("AVAL", "CHG"),
  treat = "ARM",
  rows_by = c("PARAM", "AVISIT"),
  indent = "  "
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/formatters.R
\name{format_n_pct}
\alias{format_n_pct}
\title{Format count(s) and percentage(s) (\verb{n (pct\%)})}
\usage{
format_n_pct(n, pct, .pct_digits = 1L)
}
\arguments{
\item{n}{An integer vector of counts. Length must match that of \code{pct}.}

\item{pct}{A numeric vector of percentages. Length must match that of \code{n}.}

\item{.pct_digits}{Number of decimal places to format percentage values.
Defaults to \code{1}.}
}
\value{
A character vector of strings following the format \code{"n (pct\%)"},
except if \code{n} is zero, then the format is simply \code{"0"}.
}
\description{
\code{\link[=format_n_pct]{format_n_pct()}} formats counts (\code{n}) and respective percentages (\code{pct}) as
\code{"n (pct\%)"}.
}
\examples{
# Simple cases.
format_n_pct(n = 25, pct = 18.66)
format_n_pct(n = 25, pct = 18.66, .pct_digits = 2)

# If both `n` and `pct` are zero then the format is special, i.e. simply "0".
format_n_pct(n = 0, pct = 0)

# `format_n_pct()` is vectorised over `n` and `pct` but their length must
# match.
format_n_pct(n = c(20, 50), pct = c(10, 25))

# Missing values result in empty strings in the output.
format_n_pct(n = c(20, NA), pct = c(10, 25))
format_n_pct(n = c(20, 50), pct = c(10, NA))

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/round.R
\name{round_pct}
\alias{round_pct}
\title{Rounded percentage}
\usage{
round_pct(x, digits = 1L, method = c("round", "round_sum"))
}
\arguments{
\item{x}{A numeric vector of non-negative values for which you want
percentages to be determined and rounded. Missing values (\code{NA}) are
ignored.}

\item{digits}{The number of decimal places to round to. Default is \code{0}
(integer rounding).}

\item{method}{Rounding method: \code{"round"} that uses R's base \code{\link[=round]{round()}} or
\code{"round_sum"} that uses \link[=round_sum]{dtlg::round_sum}.}
}
\value{
A numeric vector of the same length as \code{x} with rounded percentages.
}
\description{
\code{\link[=round_pct]{round_pct()}} returns the rounded percentages of \code{x} values.
}
\examples{
x <- c(1 / 3, 1 / 3, 1 / 3)

# Default method ensures precise rounding but total might not be 100\%.
round_pct(x = x)
sum(round_pct(x = x))

# You can trade off rounding precision for precision on the total with the
# method `"round_sum"`.
round_pct(x = x, method = "round_sum")
sum(round_pct(x = x, method = "round_sum"))

# Vary the number of decimal places, e.g. increase to three.
round_pct(x = x, digits = 3, method = "round_sum")

# Missing values are ignored.
x <- c(1, 2, NA)
round_pct(x = x, digits = 3)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/events.R
\name{event_count_by}
\alias{event_count_by}
\title{Summarise adverse events by arm and other grouping variables}
\usage{
event_count_by(
  dt,
  patient,
  treat,
  rows_by,
  target,
  .total_dt = dt,
  indent = nbsp(n = 4L),
  pct_dec = 1
)
}
\arguments{
\item{dt}{A \code{data.frame} or \code{data.table} containing the adverse event data
and patient-level identifiers.}

\item{patient}{A string giving the name of the patient identifier variable
(e.g., \code{"USUBJID"}).}

\item{treat}{A string giving the name of the treatment arm variable (e.g.,
\code{"ARM"}).}

\item{rows_by}{A string giving the name of the grouping variable (e.g.,
\code{"AEBODSYS"} for body system).}

\item{target}{A string giving the name of the variable to report within each
group (e.g., \code{"AEDECOD"} for preferred term).}

\item{.total_dt}{A \code{data.frame} or \code{data.table} containing the denominator
population. Defaults to \code{dt}.}

\item{indent}{A string used to indent row labels (e.g., \code{"  "} or \code{nbsp(n = 4L)}).}

\item{pct_dec}{Integer. Number of decimal places to show in percentages.
Defaults to \code{1}.}
}
\value{
A \code{data.table} with the following structure:
\itemize{
\item One row per combination of \code{rows_by} and \code{target}
\item One row per group total (\verb{Total number of events})
\item One row per patient-level total (\verb{Total number of patients with at least one event})
}

Columns include:
\itemize{
\item \code{stats}: character column with labels
\item one column per level of the \code{treat} variable, formatted as \code{"n (x\%)"}
}
}
\description{
\code{event_count_by()} creates a tabular summary of adverse events grouped by a
higher-level classification variable (e.g., system organ class), and counts
both the number of events and the number of unique patients per treatment arm.
}
\examples{
event_count_by(
  dt = adae[adae$ANL01FL == "Y"],
  patient = 'USUBJID',
  treat = 'ARM',
  rows_by = 'AEBODSYS',
  target = 'AEDECOD',
  .total_dt = adsl,
  indent = ' '
)

}
\seealso{
\code{\link[=event_count]{event_count()}}, \code{\link[=calc_stats]{calc_stats()}}, \code{\link[=total_events]{total_events()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cdisc-data.R
\docType{data}
\name{adae}
\alias{adae}
\title{Adverse Event Analysis Dataset example dataset}
\format{
An object of class \code{tbl_df} (inherits from \code{tbl}, \code{data.frame}) with 1934 rows and 92 columns.
}
\usage{
adae
}
\description{
\link{adae} is a re-export of the \link[random.cdisc.data:cadae]{random.cdisc.data::cadae} dataset, included in
\code{{dtlg}} for function usage illustration and testing.
}
\examples{
adae

}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/events.R
\name{multi_event_true}
\alias{multi_event_true}
\title{Summarise multiple AESI-like events per treatment arm}
\usage{
multi_event_true(
  dt,
  event_vars,
  patient,
  treat,
  heading,
  label = NULL,
  .total_dt = NULL,
  indent = nbsp(n = 4L),
  pct_dec = 1
)
}
\arguments{
\item{dt}{A \code{data.frame} or \code{data.table} containing the binary event flags
and subject-level data.}

\item{event_vars}{A character vector of column names (binary flags) to
summarise.}

\item{patient}{A string giving the name of the variable that uniquely
identifies each patient (e.g., \code{"USUBJID"}).}

\item{treat}{A string giving the name of the treatment variable (e.g.,
\code{"ARM"}).}

\item{heading}{A string to be shown as the first row in the output, usually a
summary descriptor such as \code{"Total number of patients with at least one"}.}

\item{label}{Optional. A character vector of the same length as \code{event_vars}
giving human-readable labels for the output table rows. If \code{NULL}, labels
are extracted from the \code{label} attribute of each variable, or fall back to
the variable name.}

\item{.total_dt}{A \code{data.frame} or \code{data.table} containing the total analysis
population (denominator). If \code{NULL}, \code{dt} is used as the denominator.}

\item{indent}{A string to indent the row labels (e.g., \code{"  "} or \code{nbsp(n = 4L)} for non-breaking spaces).}

\item{pct_dec}{An integer indicating how many decimal places to show in
percentages (default is \code{1}).}
}
\value{
A one-element list containing a \code{data.table} with one row per event
plus one header row. The first column is \code{"stats"} (row labels), and
subsequent columns are one per treatment arm, with values in \code{"n (x\%)"}
format.
}
\description{
\code{multi_event_true()} generates a summary table showing the number and
percentage of patients with at least one event across multiple binary
indicator variables (e.g., flags for adverse events of special interest).
}
\details{
Each event is counted only once per patient. This function is typically used
for summarising \emph{Adverse Events of Special Interest} (AESIs) or other derived
flags (e.g., \code{SER}, \code{FATAL}, \code{RELDSM}) that are binary (TRUE/FALSE).
}
\examples{
aesi_vars <- c(
  "FATAL", "SER", "SERWD", "SERDSM", "RELSER",
  "WD", "DSM", "REL", "RELWD", "RELDSM", "SEV"
)

heading <- "Total number of patients with at least one AE"

multi_event_true(
  dt = aesi,
  event_vars = aesi_vars,
  patient = "USUBJID",
  treat = "ARM",
  heading = heading,
  .total_dt = adsl,
  indent = "  "
)[[1]]

}
\seealso{
\code{\link[=event_count]{event_count()}}, \code{\link[=total_events]{total_events()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calc-stats.R
\name{calc_stats}
\alias{calc_stats}
\title{Calculate summary statistics for a variable}
\usage{
calc_stats(
  dt,
  target,
  treat,
  ...,
  target_name = target,
  indent = nbsp(n = 4L),
  .total_dt = NULL,
  pct_dec = 1
)
}
\arguments{
\item{dt}{A \code{data.frame} containing, at least, the variables indicated in
\code{target} and \code{treat}.}

\item{target}{Target variable passed as a string for which summary
statistics are to be calculated.}

\item{treat}{A string indicating the grouping variable, e.g. the variable
specifying the treatment population.}

\item{...}{Additional arguments passed to other methods}

\item{target_name}{Heading for the target variable as a string. Defaults to
\code{target}.}

\item{indent}{A string to be used as indentation of summary statistics
labels. Defaults to four HTML non-breaking spaces (\verb{&nbsp;}).}

\item{.total_dt}{Separate table from \code{dt} from which to derive total counts
per group.}

\item{pct_dec}{Decimal places for reported figures.}
}
\value{
A \code{data.table} of summary statistics. The format depends on the
type of the \code{target} variable:
\itemize{
\item If the \code{target} variable is categorical, i.e. type \code{character}, \code{factor}
or \code{logical} then the output is that of \code{\link[=calc_counts]{calc_counts()}}.
\item If the \code{target} variable is numeric, then the output is that of
\code{\link[=calc_desc]{calc_desc()}}.
}
}
\description{
\code{\link[=calc_stats]{calc_stats()}} calculates summary statistics for a variable on groups. This
is a generic function; note that it dispatches based on the class of
\code{target} (second argument), not \code{dt} (first argument).
}
\examples{
# Calculate summary statistics of a numeric variable, e.g. `AGE`.
calc_stats(dt = adsl, "AGE", treat = "ARM")[[1]]

# Calculate summary statistics of a categorical variable, e.g. `SEX`.
calc_stats(dt = adsl, "SEX", treat = "ARM")[[1]]

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/merge-table-lists.R
\name{merge_table_lists}
\alias{merge_table_lists}
\title{Merge a list of list-wrapped data.tables into one data.table}
\usage{
merge_table_lists(dt_l)
}
\arguments{
\item{dt_l}{A list of one-element lists, where each element is a list
containing a single \code{data.table}.}
}
\value{
A single merged \code{data.table}, row-bound from all input tables.
}
\description{
This function is typically used to combine multiple reporting tables,
each produced by \code{event_count()}, \code{total_events()}, or
\code{multi_event_true()}, into a single summary table. These intermediate tables
are often returned as one-element lists containing a \code{data.table}.
}
\details{
This helper unwraps and merges them, row-wise, to produce a
consolidated safety report table --- commonly used in clinical study reports
or data monitoring reviews.
}
\examples{
# Count deaths by treatment arm
death_table <- event_count(
  adsl,
  patient = "USUBJID",
  treat = "ARM",
  label = "Total number of deaths",
  .filters = "DTHFL == 'Y'"
)

# Count study withdrawals due to adverse events
withdrawal_table <- event_count(
  adsl,
  patient = "USUBJID",
  treat = "ARM",
  label = "Total number of patients withdrawn from study due to an AE",
  .filters = "DCSREAS == 'ADVERSE EVENT'"
)

# Count patients with at least one adverse event
patients_with_ae_table <- event_count(
  adae,
  patient = "USUBJID",
  treat = "ARM",
  label = "Total number of patients with at least one AE"
)

# Count total number of adverse events (not patients)
total_ae_events_table <- total_events(
  dt = adae,
  treat = "ARM",
  label = "Total number of AEs"
)

# Summarise AESIs (e.g., serious, related, severe, etc.)
aesi_vars <- c("FATAL", "SER", "SERWD", "SERDSM", "RELSER",
               "WD", "DSM", "REL", "RELWD", "RELDSM", "SEV")

aesi_table <- multi_event_true(
  dt = aesi,
  event_vars = aesi_vars,
  patient = "USUBJID",
  treat = "ARM",
  heading = "Total number of patients with at least one",
  .total_dt = adsl,
  indent = "  "
)

# Combine all safety tables into a single summary table
safety_summary <- merge_table_lists(list(
  patients_with_ae_table,
  total_ae_events_table,
  death_table,
  withdrawal_table,
  aesi_table
))

safety_summary

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/events.R
\name{total_events}
\alias{total_events}
\title{Count total events}
\usage{
total_events(dt, treat, label)
}
\arguments{
\item{dt}{A \code{data.frame} containing, at least, the variable indicated in
\code{treat}.}

\item{treat}{A string indicating the grouping variable, e.g. the variable
specifying the treatment population.}

\item{label}{A string to be used as label in the output reporting table. This
should be a text descriptive of the event being counted.}
}
\value{
A list wrapping a one-row \code{data.table} of \code{1 + n} variables, where
\code{n} is the number of levels in \code{treat}. First variable is \code{stats},
\code{character} type, whose value is the argument passed in as \code{label}.
Following variables are of \code{integer} type and provide the counts.
}
\description{
\code{\link[=total_events]{total_events()}} counts the number of observations in \code{dt} in each group
defined by \code{treat} levels. Counts are returned in wide format, i.e. one
column per level in \code{treat}.
}
\examples{
# In the absence of pre-filtering, `total_events()`, actually, just counts
# observations in `dt`.
total_events(dt = adsl, treat = "ARM", label = "Subjects")[[1]]

# If `dt` is pre-filtered, e.g. with a condition matching an event, then
# `total_events()` can be used to (effectively) count events.
total_events(dt = adsl[adsl$DTHFL == "Y"], treat = "ARM", label = "Deaths")[[1]]

# Another example using the complement predicate condition.
total_events(dt = adsl[adsl$DTHFL == "N"], treat = "ARM", label = "Lives")[[1]]

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AET01_table.R
\name{AET01_table}
\alias{AET01_table}
\title{Generate Core Safety Tables for Clinical Study Reports}
\usage{
AET01_table(
  adsl,
  adae,
  patient_var,
  treat_var,
  aesi_vars,
  aesi_heading = "Total number of patients with at least one",
  indent = "  "
)
}
\arguments{
\item{adsl}{A subject-level dataset (typically ADaM ADSL).}

\item{adae}{A dataset of adverse events, preprocessed with AESI flags.}

\item{patient_var}{A string indicating the subject identifier variable (e.g., \code{"USUBJID"}).}

\item{treat_var}{A string indicating the treatment arm variable (e.g., \code{"ARM"}).}

\item{aesi_vars}{A character vector of binary AESI flags in \code{adae}.}

\item{aesi_heading}{Optional character string used as a heading in the AESI block.}

\item{indent}{A string used to indent AESI row labels (default is 2 spaces).}
}
\value{
A merged \code{data.table} summarising the main safety outcomes.
}
\description{
\code{\link[=AET01_table]{AET01_table()}} produces and combines the main safety summary tables
typically found in Section 14.3.1 of a Clinical Study Report (CSR). It
calculates patient counts and event totals for deaths, AE-related
withdrawals, total AEs, and adverse events of special interest (AESIs).
}
\examples{
AET01_summary <- AET01_table(
  adsl = adsl,
  adae = aesi,
  patient_var = "USUBJID",
  treat_var = "ARM",
  aesi_vars = c("FATAL", "SER", "SERWD", "SERDSM", "RELSER",
                "WD", "DSM", "REL", "RELWD", "RELDSM", "SEV")
)
print(AET01_summary)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tern.R
\name{tern_summary_table}
\alias{tern_summary_table}
\title{Create a clinical reporting table with tern/rtables}
\usage{
tern_summary_table(dt, target, treat, target_name = target)
}
\arguments{
\item{dt}{A \code{data.frame} containing, at least, the variables indicated in
\code{target} and \code{treat}.}

\item{target}{Target variable passed as a string for which summary
statistics are to be calculated.}

\item{treat}{A string indicating the grouping variable, e.g. the variable
specifying the treatment population.}

\item{target_name}{Heading for the target variable as a string. Defaults to
\code{target}.}
}
\value{
A \code{data.table} of summary statistics. The format depends on the
type of the \code{target} variable:
\itemize{
\item If the \code{target} variable is categorical, i.e. type \code{character}, \code{factor}
or \code{logical} then the output is that of \code{\link[=calc_counts]{calc_counts()}}.
\item If the \code{target} variable is numeric, then the output is that of
\code{\link[=calc_desc]{calc_desc()}}.
}
}
\description{
\code{\link[=tern_summary_table]{tern_summary_table()}} is a convenience wrapper around \code{{rtables}} and
\code{{tern}} commands to generate a clinical reporting summary statistics tables
whilst using a similar interface as \code{\link[=summary_table]{summary_table()}}. This can be helpful
for side by side comparisons of the two functions.
}
\examples{
dmg_vars <- c("AGE", "RACE", "ETHNIC")
dmg_var_lbls <- c("Age (yr)", "Race", "Ethnicity")

# Demographics table (DMT01)
tern_summary_table(
  adsl,
  target = dmg_vars,
  treat = 'ARM',
  target_name = dmg_var_lbls
)

# Demographics table (DMT01) with continuous variable (e.g., BMRKR1)
tern_summary_table(
  adsl,
  target = c(dmg_vars, "BMRKR1"),
  treat = 'ARM',
  target_name = c(dmg_var_lbls, "Biomarker 1")
)

}
\seealso{
\code{\link[=summary_table]{summary_table()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calc-stats.R
\name{calc_desc}
\alias{calc_desc}
\title{Calculate descriptive summary statistics for a numeric variable}
\usage{
calc_desc(
  dt,
  target,
  target_name = target,
  treat,
  indent = nbsp(n = 4L),
  pct_dec = 1,
  inc_missing = TRUE
)
}
\arguments{
\item{dt}{A \code{data.frame} containing, at least, the variables indicated in
\code{target} and \code{treat}.}

\item{target}{Target variable passed as a string for which summary
statistics are to be calculated.}

\item{target_name}{Heading for the target variable as a string. Defaults to
\code{target}.}

\item{treat}{A string indicating the grouping variable, e.g. the variable
specifying the treatment population.}

\item{indent}{A string to be used as indentation of summary statistics
labels. Defaults to four HTML non-breaking spaces (\verb{&nbsp;}).}

\item{pct_dec}{Decimal places for reported figures.}

\item{inc_missing}{Toggle for including the "Missing" row:
\describe{
\item{\code{TRUE}}{(default) The Missing row is always displayed}
\item{\code{NA}}{The Missing row is only displayed if any missing values are present}
\item{\code{FALSE}}{The Missing row is never included in the table}
}}
}
\value{
A list containing a \code{data.table} formatted as follows:
\itemize{
\item First column is named \code{stats} and contains the target variable name
indicated in \code{target} in the first row. Subsequent rows contain the
summarised statistics labels.
\item Other columns are for the levels of the grouping variable (\code{treat}).
\item All columns are of character type.
}

This table is structured for easy integration with Shiny output widgets.
}
\description{
\code{\link[=calc_desc]{calc_desc()}} summarises a numeric variable (\code{target}) by another (\code{treat}) and
reports summary statistics in clinical trial reporting format. The following
statistics are calculated for \code{target}, per group, i.e. by variable \code{treat}
levels:
\itemize{
\item \code{n}: number of observations
\item \code{Mean (SD)}: mean and standard deviation of \code{target}
\item \code{Median}: median of \code{target}
\item \verb{Min, Max}: minimum and maximum of \code{target}
\item \code{Missing}: number of missing \code{target} values
}
}
\examples{
# Calculate summary statistics for the age of the subjects in each region.
calc_desc(dt = adsl, "AGE", treat = "REGION1")[[1]]

# Calculate summary statistics for biomarker 1 in each of the three arms
# (`ARM`).
calc_desc(dt = adsl, "BMRKR1", treat = "ARM")[[1]]

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary-table.R
\name{summary_table_by}
\alias{summary_table_by}
\title{Create a summary table using multiple rows for grouping on one target column}
\usage{
summary_table_by(
  dt,
  target,
  treat,
  rows_by,
  indent = nbsp(n = 4L),
  .total_dt = dt,
  pct_dec = 1,
  treat_order = NULL,
  skip_absent = TRUE,
  sep = "."
)
}
\arguments{
\item{dt}{A \code{data.frame} containing, at least, the variables indicated in
\code{target} and \code{treat}.}

\item{target}{Target variable passed as a string for which summary
statistics are to be calculated.}

\item{treat}{A string indicating the grouping variable, e.g. the variable
specifying the treatment population.}

\item{rows_by}{string, grouping variable to split events by.}

\item{indent}{A string to be used as indentation of summary statistics
labels. Defaults to four HTML non-breaking spaces (\verb{&nbsp;}).}

\item{.total_dt}{Separate table from \code{dt} from which to derive total counts
per group.}

\item{pct_dec}{Decimal places for reported figures.}

\item{treat_order}{Customise the column order of the output table.}

\item{skip_absent}{Whether to ignore variables passed in \code{treat_order} that
are absent from \code{dt}. Default is \code{TRUE}; \code{FALSE} will throw an error in
case there are missing variables.}

\item{sep}{character string to separate the terms}
}
\value{
The same output as \code{\link[=summary_table]{summary_table()}} except that folded by variables
indicated in \code{rows_by}.
}
\description{
Create a summary table using multiple rows for grouping on one target column
}
\examples{
summary_table_by(
  adlb,
  target = "AVAL",
  treat = "ARM",
  rows_by = c("PARAM","AVISIT")
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cdisc-data.R
\docType{data}
\name{adlb}
\alias{adlb}
\title{ADaM Basic Data Structure (BDS) example dataset}
\format{
An object of class \code{data.table} (inherits from \code{data.frame}) with 8400 rows and 102 columns.
}
\usage{
adlb
}
\description{
\link{adlb} is a re-export of the \link[random.cdisc.data:cadlb]{random.cdisc.data::cadlb} dataset, included in
\code{{dtlg}} for function usage illustration and testing.
}
\examples{
adlb

}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cdisc-data.R
\docType{data}
\name{adsl}
\alias{adsl}
\title{Subject-Level Analysis Dataset (ADSL) example dataset}
\format{
An object of class \code{tbl_df} (inherits from \code{tbl}, \code{data.frame}) with 400 rows and 55 columns.
}
\usage{
adsl
}
\description{
\link{adsl} is a re-export of the \link[random.cdisc.data:cadsl]{random.cdisc.data::cadsl} dataset, included in
\code{{dtlg}} for function usage illustration and testing.
}
\examples{
adsl

}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tern.R
\name{tern_AET01_table}
\alias{tern_AET01_table}
\title{Generate Core Safety Tables (CSR Section 14.3.1) using \code{tern}/\code{rtables}}
\usage{
tern_AET01_table(
  adsl,
  adae,
  patient_var,
  treat_var,
  aesi_vars,
  aesi_heading = "Total number of patients with at least one",
  indent = "  "
)
}
\arguments{
\item{adsl}{A subject-level dataset (typically ADaM ADSL).}

\item{adae}{A dataset of adverse events, preprocessed with AESI flags.}

\item{patient_var}{A string indicating the subject identifier variable (e.g.,
\code{"USUBJID"}).}

\item{treat_var}{A string indicating the treatment arm variable (e.g., \code{"ARM"}).}

\item{aesi_vars}{A character vector of binary AESI flags in \code{adae}.}

\item{aesi_heading}{Ignored (included for interface compatibility).}

\item{indent}{Ignored (included for interface compatibility).}
}
\value{
A \code{TableTree} object from the \code{rtables} package.
}
\description{
\code{\link[=tern_AET01_table]{tern_AET01_table()}} produces a consolidated safety summary table using
\code{rtables} and \code{tern}. It mirrors the output and interface of
\code{\link[=AET01_table]{AET01_table()}}, generating standard adverse event summaries (e.g. death,
withdrawal, AESIs) for Clinical Study Reports (CSR) Section 14.3.1.
}
\details{
The function returns a single formatted \code{rtables} table summarising core
safety endpoints by treatment arm.
}
\examples{
tern_AET01_table(
  adsl = adsl,
  adae = aesi,
  patient_var = "USUBJID",
  treat_var = "ARM",
  aesi_vars = c("FATAL", "SER", "SERWD", "SERDSM", "RELSER",
                "WD", "DSM", "REL", "RELWD", "RELDSM", "SEV")
)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cross_tab_to_obsv_tab.R
\name{cross_tab_to_obsv_tab}
\alias{cross_tab_to_obsv_tab}
\title{Convert a contingency table to a long-format observation-level data frame}
\usage{
cross_tab_to_obsv_tab(cross_tab, strings_as_factors = TRUE)
}
\arguments{
\item{cross_tab}{A two-way or multi-way contingency table (\code{matrix} or
\code{table}) with named \code{dimnames}. Each combination of factor levels is
assumed to represent a count of occurrences.}

\item{strings_as_factors}{Should character columns in the output
be converted to factors?}
}
\value{
A \code{data.frame} in long format with one row per implied observation
and one column per dimension of the input table.
}
\description{
\code{\link[=cross_tab_to_obsv_tab]{cross_tab_to_obsv_tab()}} expands a contingency table or matrix of counts
into a long-format data frame where each row represents one observation. The
output contains one column per dimension of the input, with repeated rows
according to the frequency counts.
}
\examples{
dim_names <- list(Sex = c("Male", "Female"),
                  Response = c("Yes", "No"))
cross_tab <- matrix(c(2, 1, 3, 4), nrow = 2, dimnames = dim_names)
cross_tab_to_obsv_tab(cross_tab)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cdisc-data.R
\docType{data}
\name{aesi}
\alias{aesi}
\title{Adverse Events of Special Interest (AESI) example dataset}
\format{
A \code{data.frame} with a subset of rows from \code{cadae} and additional
derived columns including:
\describe{
\item{FATAL}{Logical flag for fatal AEs (\code{AESDTH == "Y"}).}
\item{SEV}{Logical flag for severe AEs (\code{AESEV == "SEVERE"}).}
\item{SER}{Logical flag for serious AEs (\code{AESER == "Y"}).}
\item{SERWD}{Serious AE leading to withdrawal (\code{AESER == "Y" & AEACN == "DRUG WITHDRAWN"}).}
\item{SERDSM}{Serious AE leading to dose modification/interruption.}
\item{RELSER}{Serious and related AE.}
\item{WD}{AE leading to withdrawal.}
\item{DSM}{AE leading to dose modification/interruption.}
\item{REL}{Related AE.}
\item{RELWD}{Related AE leading to withdrawal.}
\item{RELDSM}{Related AE leading to dose modification/interruption.}
}
}
\usage{
aesi
}
\description{
\link{aesi} is a modified version of the \link[random.cdisc.data:cadae]{random.cdisc.data::cadae} dataset,
filtered to include only analysis-flagged records (\code{ANL01FL == "Y"}) and
extended with binary indicator variables corresponding to adverse events of
special interest (AESIs).
}
\details{
These derived flags include seriousness, severity, fatality, relatedness, and
treatment consequence (e.g., dose modification or withdrawal), and are used to
illustrate key safety summaries in clinical reporting.

Each derived variable is labeled using \code{\link[=with_label]{with_label()}} for compatibility
with tabulation functions.

This dataset is included in \code{{dtlg}} to support function testing, usage
examples, and reproducible safety analyses.
}
\examples{
aesi

}
\seealso{
\link[random.cdisc.data:cadae]{random.cdisc.data::cadae}, \code{\link[=multi_event_true]{multi_event_true()}}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/with_label.R
\name{label}
\alias{label}
\title{Retrieve the label of an object}
\usage{
label(x)
}
\arguments{
\item{x}{An R object.}
}
\value{
The label attribute (string) associated with object passed in \code{x} or
\code{NULL} if the label attribute does not exist.
}
\description{
\code{\link[=label]{label()}} gets the attached label to an object.
}
\examples{
label(1)
label(with_label(1, "my label"))

}
\seealso{
\code{\link[=with_label]{with_label()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/indentation.R
\name{indent}
\alias{indent}
\title{Add indentation to strings}
\usage{
indent(x, n = 4L, indentation = nbsp(n = n))
}
\arguments{
\item{x}{A character vector of strings to be \emph{indented}.}

\item{n}{Number of non-breaking spaces to use as indentation. If \code{n = 0} then
no indentation is performed.}

\item{indentation}{As an alternative to the number of spaces \code{n} you may
pass the actual sequence of HTML non-breaking spaces as a string to this
parameter, or any other string for that matter. Defaults to an indentation
of four spaces (\code{n = 4}).}
}
\value{
A character vector of the same length as \code{x}.
}
\description{
\code{\link[=indent]{indent()}} prefixes a string with a sequence of HTML non-breaking spaces,
to effectively work as indentation.
}
\examples{
# Default is to indent by four non-breaking spaces.
indent("Mean")

# Choose a different indentation level.
indent("Mean", n = 2L)

# `indent()` is vectorised over `x`
indent(c("Mean", "Median", "Max, Min", "Missing"))

}
\keyword{internal}
